/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file battleship_menu.c 
 *
 *  Handels menus.
 *
 *  These files are responsible for the game logic related to menus. They
 *  take care of the movement in and between menus as well as toggling
 *  some variables connect to the menu.
 * 
 */
#include <stddef.h>

#include "battleship_menu.h"
#include "battleship_logic.h"
#include "battleship_model.h"
#include "battleship_rocket.h"
#include "battleship_sound.h"
#include "lsystem.h"

/** Main menu, New menu item */
static struct battleship_menu main_new;
/** Main menu, New one player menu item */
static struct battleship_menu main_onep;
/** Main menu, New two player menu item */
static struct battleship_menu main_twop;
/** Main menu, Demo menu item */
static struct battleship_menu main_demo;
/** Main menu, Quit menu item */
static struct battleship_menu main_quit;
/** Main menu, Options menu item */
static struct battleship_menu main_options;
/** Main menu, items */
static struct battleship_menu *main_items[] = 
{
	&main_new, 
	&main_onep, 
	&main_twop, 
	&main_demo, 
	&main_options,
	&main_quit
};


/** fire sub menu */
static struct battleship_menu fire_menu;
/** fire sub menu, fire enabled */
static struct battleship_menu fire_enabled;
/** fire sub menu, fire fire length */
static struct battleship_menu fire_len;
/** fire sub menu, fire fire width */
static struct battleship_menu fire_width;
/** fire sub menu, fire fire colour */
static struct battleship_menu fire_colour;
/** fire sub menu, items */
static struct battleship_menu *fire_items[] = 
{
	&fire_enabled,
	&fire_len,
	&fire_width,
	&fire_colour,
};

/** rocket sub menu */
static struct battleship_menu rocket_menu;
/** rocket sub menu, rocket enabled */
static struct battleship_menu rocket_enabled;
/** rocket sub menu, rocket path drawing enabled */
static struct battleship_menu rocket_path;
/** rocket sub menu, rocket speed */
static struct battleship_menu rocket_speed;
/** rocket sub menu, rocket size */
static struct battleship_menu rocket_size;
/** rocket sub menu, items */
static struct battleship_menu *rocket_items[] = 
{
	&rocket_enabled,
	&rocket_path,
	&rocket_size,
	&rocket_speed,
	&fire_menu,
};

/** Animation sub menu */
static struct battleship_menu animation;
/** Animation sub menu, Transition item */
static struct battleship_menu animation_trans;
/** Animation sub menu, AI item */
static struct battleship_menu animation_ai;
/** Animation sub menu, Msg item */
static struct battleship_menu animation_msg;
/** Animation sub menu, items */
static struct battleship_menu *animation_items[] = 
{
	&animation_trans,
	&animation_ai,
	&animation_msg,
	&rocket_menu,
};

/** Game options sub menu */
static struct battleship_menu gameoptions;
/** Options sub menu, Player 1 auto place menu item */
static struct battleship_menu gameoptions_p1_auto;
/** Options sub menu, Player 2 auto place menu item */
static struct battleship_menu gameoptions_p2_auto;
/** Options sub menu, Player 1 AI menu item */
static struct battleship_menu gameoptions_p1_ai;
/** Options sub menu, Player 2 AI menu item */
static struct battleship_menu gameoptions_p2_ai;
/** Options sub menu, Player 1 name menu item */
static struct battleship_menu gameoptions_p1_name;
/** Options sub menu, Player 2 name menu item */
static struct battleship_menu gameoptions_p2_name;
/** Games Options sub menu, items */
static struct battleship_menu *gameoptions_items[] = 
{
	&gameoptions_p1_name, 
	&gameoptions_p1_ai, 
	&gameoptions_p1_auto, 
	NULL,
	&gameoptions_p2_name, 
	&gameoptions_p2_ai,
	&gameoptions_p2_auto,
};

/** Fog sub menu */
static struct battleship_menu fog_menu;
/** Fog sub menu, Type item */
static struct battleship_menu fog_type;
/** Fog sub menu, Regenerate item */
static struct battleship_menu fog_regen;
/** Fog sub menu, P1 fog enabled menu item */
static struct battleship_menu fog_p1;
/** Fog sub menu. P2 fog enabled menu item */
static struct battleship_menu fog_p2;
/** Fog sub menu, items */
static struct battleship_menu *fog_items[] = 
{
	&fog_type,
	&fog_regen,
	&fog_p1,
	&fog_p2,
};

/** Shadow sub menu */
static struct battleship_menu shadow_menu;
/** Shadow sub menu, Enabled */
static struct battleship_menu shadow_enabled;
/** Shadow sub menu, Table shadows */
static struct battleship_menu shadow_table;
/** Shadow sub menu, Shadows on back wall */
static struct battleship_menu shadow_back_wall;
/** Shadow sub menu, Shadows on front wall*/
static struct battleship_menu shadow_front_wall;
/** Shadow sub menu. Shadows on floor */
static struct battleship_menu shadow_floor;
/** Shadow sub menu, items */
static struct battleship_menu *shadow_items[] = 
{
	&shadow_enabled,
	&shadow_table,
	&shadow_front_wall,
	&shadow_back_wall,
	&shadow_floor,
};

/** lsys sub menu */
static struct battleship_menu lsys_menu;
/** lsys sub menu, Enabled item */
static struct battleship_menu lsys_enabled;
/** lsys sub menu, Type item */
static struct battleship_menu lsys_type;
/** lsys sub menu, length item */
static struct battleship_menu lsys_length;
/** L-System sub menu, items */
static struct battleship_menu *lsys_items[] = 
{
	&lsys_enabled,
	&lsys_type,
	&lsys_length,
};

/** Options sub menu, textures option */
static struct battleship_menu options_texture;
/** Options sub menu, sound option */
static struct battleship_menu options_sound;
/** Options sub menu, test option */
static struct battleship_menu options_test;
/** Options sub menu, quit menu item */
static struct battleship_menu options_quit;
/** Optiones sub menu, items */
static struct battleship_menu *main_options_items[] = 
{
	&gameoptions,
	&animation,
	&fog_menu,
	&lsys_menu,
	&shadow_menu,
	&options_texture,
	&options_sound,
//	&options_test
};
/** In game options menu items */
static struct battleship_menu *ingame_options_items[] = 
{
	&gameoptions,
	&animation,
	&fog_menu,
	&lsys_menu,
	&shadow_menu,
	&options_texture,
	&options_sound,
//	&options_test,
	&options_quit
};

/** Toggle function for new game option, just a wrapper that calls
 *  battleship_logic_new()
 */
static void newgame(struct battleship_menu *m, char dir);
/** Toggle function for onep game option, just a wrapper that calls
 *  battleship_logic_onep()
 */
static void onep(struct battleship_menu *m, char dir);
/** Toggle function for twop game option, just a wrapper that calls
 *  battleship_logic_twop()
 */
static void twop(struct battleship_menu *m, char dir);
/** Toggle function for demo option, just a wrapper that calls
 *  battleship_logic_demo()
 */
static void demogame(struct battleship_menu *m, char dir);
/** Toggle function for quit option, just a wrapper that calls
 *  battleship_logic_quit()
 */
static void quitgame(struct battleship_menu *m, char dir);
/** Toggle function for AI type option, switches between AI types */
static void toggle_ai(struct battleship_menu *m, char dir);
/** Toggle function for quit in options menu, returns to main menu */
static void quit_curr_game(struct battleship_menu *m, char dir);
/** Generic toggle function for on/off type menu items */
static void toggle_bool(struct battleship_menu *m, char dir);
/** Toggle function for test type option, switches between test types */
static void toggle_test(struct battleship_menu *m, char dir);
/** Toggle function for rocket type option, switches between rocket paths */
static void toggle_rocket(struct battleship_menu *m, char dir);
/** Toggle function for fog type option, switches between fog types */
static void toggle_fog(struct battleship_menu *m, char dir);
/** Toggle function for fog regen, cause fog to be regenerated */
static void toggle_foggen(struct battleship_menu *m, char dir);
/** Toggle function for name type option, activates name selector */
static void toggle_name(struct battleship_menu *m, char dir);
/** Toggle function for lsystem type option */
static void toggle_lsys_type(struct battleship_menu *m, char dir);
/** Toggle function for lsystem length option */
static void toggle_lsys_length(struct battleship_menu *m, char dir);
/** Toggle rocket size */
static void toggle_rocket_size(struct battleship_menu *m, char dir);
/** Toggle rocket size */
static void toggle_rocket_speed(struct battleship_menu *m, char dir);
/** Toggle fire length */
static void toggle_fire_length(struct battleship_menu *m, char dir);
/** Toggle fire width */
static void toggle_fire_width(struct battleship_menu *m, char dir);
/** Toggle fire colour */
static void toggle_fire_colour(struct battleship_menu *m, char dir);

/** Macro for making a menu */
#define MENU_ITEM(menu, mname, val, act) \
	tempMenu = &menu;			\
	tempMenu->name = mname;		\
	tempMenu->parent = NULL;	\
	tempMenu->item = NULL;		\
	tempMenu->size = 0;			\
	tempMenu->svalue = NULL;	\
	tempMenu->value = val;		\
	tempMenu->action = act;

/** Macro for making a menu item */
#define MENU(menu, mname, items) 										\
	tempMenu = &menu;													\
	tempMenu->name = mname;												\
	tempMenu->parent = NULL;											\
	tempMenu->item = items;												\
	tempMenu->size = sizeof(items)/sizeof(struct battleship_menu *); 	\
	tempMenu->svalue = NULL;	\
	tempMenu->value = NULL;		\
	tempMenu->action = NULL;

void
battleship_menu_init()
{
	struct battleship_menu *tempMenu;


	//Main menu
	MENU_ITEM(main_new, "New Game", NULL, newgame);
	MENU_ITEM(main_onep, "1 Player", NULL, onep);
	MENU_ITEM(main_twop, "2 Player", NULL, twop);
	MENU_ITEM(main_demo, "Demo Mode", NULL, demogame);	
	MENU_ITEM(main_quit, "Quit", NULL, quitgame);	
	MENU(main_options, "Options", main_options_items);
	MENU(main_menu, BATTLESHIP_NAME " " BATTLESHIP_VERSION, main_items);

	//Options menus
	MENU_ITEM(options_texture, "Textures", &do_textures, toggle_bool);
	MENU_ITEM(options_sound, "Sound", &do_sound, toggle_bool);
	MENU_ITEM(options_test, "Tests", &do_test, toggle_test);
	MENU_ITEM(options_quit, "Main Menu", NULL, quit_curr_game);
	MENU(options_menu, "Options", ingame_options_items); 

	//Options Animation submenu
	MENU_ITEM(animation_trans, "Game Board", 
			&do_trans_animation, toggle_bool);
	MENU_ITEM(animation_ai, "AI", &do_ai_animation, toggle_bool);
	MENU_ITEM(animation_msg, "Message", &do_msg_animation, 
			toggle_bool);
	MENU(animation, "Animation", animation_items); 
	
	//Rocket Animation sub menu
	MENU_ITEM(rocket_enabled, "Path", &game_rocket.type, 
			toggle_rocket);
	MENU_ITEM(rocket_path, "Draw Path", &game_rocket.show_path, 
			toggle_bool);
	MENU_ITEM(rocket_size, "Size", &game_rocket.size, 
			toggle_rocket_size);
	MENU_ITEM(rocket_speed, "Speed", &game_rocket.speed, 
			toggle_rocket_speed);
	MENU(rocket_menu, "Rocket", rocket_items); 
	
	//Rockte fire submenu
	MENU_ITEM(fire_enabled, "Enabled", &game_rocket.draw_fire, 
			toggle_bool);
	MENU_ITEM(fire_len, "Length", &game_rocket.len, 
			toggle_fire_length);
	MENU_ITEM(fire_width, "Width", &game_rocket.width, 
			toggle_fire_width);
	MENU_ITEM(fire_colour, "Colour", &game_rocket.colour, 
			toggle_fire_colour);
	MENU(fire_menu, "Fire", fire_items); 

	//Options game submenu
	MENU_ITEM(gameoptions_p1_name, "P1 Name", player[0].name, toggle_name);
	MENU_ITEM(gameoptions_p1_ai, "P1 AI", &player[0].ai, toggle_ai);
	MENU_ITEM(gameoptions_p1_auto, "P1 Auto Place", &player[0].auto_place, 
				toggle_bool);
	MENU_ITEM(gameoptions_p2_name, "P2 Name", player[1].name, toggle_name);
	MENU_ITEM(gameoptions_p2_ai, "P2 AI", &player[1].ai, toggle_ai);
	MENU_ITEM(gameoptions_p2_auto, "P2 Auto Place", &player[1].auto_place, 
				toggle_bool);
	MENU(gameoptions, "Player Options", gameoptions_items);
	
	//Fog submenu
	MENU_ITEM(fog_type, "Fog density", &do_fog, toggle_fog);
	MENU_ITEM(fog_regen, "Regenerate", NULL, toggle_foggen);
	MENU_ITEM(fog_p1, "P1 Hide Ships", &player[0].fog, toggle_bool);
	MENU_ITEM(fog_p2, "P2 Hide Ships", &player[1].fog, toggle_bool);
	MENU(fog_menu, "Fog", fog_items);
	
	//lsys submenu
	MENU_ITEM(lsys_enabled, "Enabled", &game_lsys.enabled, toggle_bool);
	MENU_ITEM(lsys_type, "Pattern", &game_lsys.type, toggle_lsys_type);
	MENU_ITEM(lsys_length, "Size", &game_lsys.length, toggle_lsys_length);
	MENU(lsys_menu, "L-Systems", lsys_items);

	//Shadow submenu
	MENU_ITEM(shadow_enabled, "Enabled", &do_shadows[0], toggle_bool);
	MENU_ITEM(shadow_table, "Table", &do_shadows[1], toggle_bool);
	MENU_ITEM(shadow_back_wall, "P2 Wall", &do_shadows[2], toggle_bool);
	MENU_ITEM(shadow_front_wall, "P1 Wall", &do_shadows[3], toggle_bool);
	MENU_ITEM(shadow_floor, "Floor", &do_shadows[4], toggle_bool);
	MENU(shadow_menu, "Shadows", shadow_items);

	curr_menu_sel = 0;
	curr_menu = NULL;
}

void 
battleship_menu_up()
{
	do 
	{
		if (curr_menu_sel == 0)
			curr_menu_sel = curr_menu->size - 1;
		else
			--curr_menu_sel;
	} while (!curr_menu->item[curr_menu_sel]);
}

void 
battleship_menu_down()
{
	do 
	{
		if (curr_menu_sel == curr_menu->size - 1)
			curr_menu_sel = 0;
		else
			++curr_menu_sel;
	} while (!curr_menu->item[curr_menu_sel]);
}

void 
battleship_menu_left()
{
	if (curr_menu->item[curr_menu_sel]->action)
		curr_menu->item[curr_menu_sel]->action(
				                curr_menu->item[curr_menu_sel], -1);
}

void 
battleship_menu_right()
{
	if (curr_menu->item[curr_menu_sel]->action)
		curr_menu->item[curr_menu_sel]->action(
				                curr_menu->item[curr_menu_sel], 1);
}

void 
battleship_menu_toggle()
{
	if (curr_menu->item[curr_menu_sel]->action)
	{
		curr_menu->item[curr_menu_sel]->action(
				curr_menu->item[curr_menu_sel], 0);
	}
	else if (curr_menu->item[curr_menu_sel]->size > 0)
	{
		battleship_menu_load(curr_menu->item[curr_menu_sel]);
	}
}

void 
battleship_menu_cancel()
{
	curr_menu = curr_menu->parent;
	curr_menu_sel = 0;
}

void
battleship_menu_load(struct battleship_menu *menu)
{
	menu->parent = curr_menu;
	curr_menu = menu;
	curr_menu_sel = 0;

	//update string value of menu items
	size_t i;
	for (i = 0; i < curr_menu->size; i++)
	{
		if (!curr_menu->item[i]) continue;

		if (curr_menu->item[i]->action &&
			curr_menu->item[i]->value)
		{
			curr_menu->item[i]->action(
				curr_menu->item[i], 0);
		}
	}
}

static void
newgame(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_new();
}

static void
onep(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_onep();
}

static void
twop(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_twop();
}

static void
demogame(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_demo();
}

static void
quitgame(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_quit();
}

static void
toggle_bool(struct battleship_menu *m, char dir)
{
	if (dir != 0)
		(*(bool *) m->value) = !(*(bool *) m->value);
	if (*((bool *) m->value))
		m->svalue = "On ";
	else
		m->svalue = "Off";
}

static void
toggle_ai(struct battleship_menu *m, char dir)
{
	int val = (int) (*(enum battleship_aitype *) m->value);
	val = (val + NUM_AI_TYPES + dir) % NUM_AI_TYPES;
	(*(enum battleship_aitype *) m->value) = (enum battleship_aitype) val; 	
	m->svalue = (char *) battleship_aitype_s[val];
}

static void
quit_curr_game(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		battleship_logic_restart();
}

static void
toggle_test(struct battleship_menu *m, char dir)
{
	int val = (int) (*(enum battleship_testtype *) m->value);
	val = (val + NUM_TEST_TYPES + dir) % NUM_TEST_TYPES;
	(*(enum battleship_testtype *) m->value) = (enum battleship_testtype) val; 	
	m->svalue = (char *) battleship_testtype_s[val];
}

static void
toggle_rocket(struct battleship_menu *m, char dir)
{
	int val = (int) (*(enum battleship_rockettype *) m->value);
	val = (val + NUM_ROCKET_TYPES + dir) % NUM_ROCKET_TYPES;
	(*(enum battleship_rockettype *) m->value)=(enum battleship_rockettype)val;	
	m->svalue = (char *) battleship_rockettype_s[val];
}

static void
toggle_fog(struct battleship_menu *m, char dir)
{
	int val = (int) (*(enum battleship_fogtype *) m->value);
	val = (val + NUM_FOG_TYPES + dir) % NUM_FOG_TYPES;
	(*(enum battleship_fogtype *) m->value) = (enum battleship_fogtype) val; 	
	m->svalue = (char *) battleship_fogtype_s[val];
}

static void
toggle_foggen(struct battleship_menu *m, char dir)
{
	(void) m;
	if (dir == 0)
		do_fog = FOG_REGEN;
}

static void
toggle_name(struct battleship_menu *m, char dir)
{
	if (dir != 0)
	{	
		name_selector.enabled = true;
		name_selector.x = 0;
		name_selector.y = 0;
		name_selector.name = (char *) m->value;
	}
	m->svalue = (char *) m->value;
}

static void
toggle_lsys_type(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = (val + NUM_LSYSTEM_TESTS + dir) % NUM_LSYSTEM_TESTS;
	(*(unsigned char *) m->value) = (unsigned char) val; 
	m->svalue = (char *) lsystem_test_s[val];
}

static void
toggle_lsys_length(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = val + dir;
	if (val > MAX_LSYSTEM_LENGTH)
		val = MAX_LSYSTEM_LENGTH;
	if (val < MIN_LSYSTEM_LENGTH)
		val = MIN_LSYSTEM_LENGTH;
	(*(unsigned char *) m->value) = (unsigned char) val; 	
	snprintf(game_lsys.slength, 4, "%i", val);
	m->svalue = game_lsys.slength;
}

static void
toggle_rocket_size(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = val + dir;
	if (val > MAX_ROCKET_SIZE)
		val = MAX_ROCKET_SIZE;
	if (val < MIN_ROCKET_SIZE)
		val = MIN_ROCKET_SIZE;
	(*(unsigned char *) m->value) = (unsigned char) val; 	
	snprintf(game_rocket.ssize, 3, "%i", val);
	m->svalue = game_rocket.ssize;
	battleship_rocket_update(); 
}

static void
toggle_rocket_speed(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = val + dir;
	if (val > MAX_ROCKET_SPEED)
		val = MAX_ROCKET_SPEED;
	if (val < MIN_ROCKET_SPEED)
		val = MIN_ROCKET_SPEED;
	(*(unsigned char *) m->value) = (unsigned char) val; 	
	snprintf(game_rocket.sspeed, 3, "%i", val);
	m->svalue = game_rocket.sspeed;
}

static void
toggle_fire_length(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = val + dir;
	if (val > MAX_FIRE_LENGTH)
		val = MAX_FIRE_LENGTH;
	if (val < MIN_FIRE_LENGTH)
		val = MIN_FIRE_LENGTH;
	(*(unsigned char *) m->value) = (unsigned char) val; 	
	snprintf(game_rocket.slen, 3, "%i", val - MIN_FIRE_LENGTH + 1);
	m->svalue = game_rocket.slen;
	battleship_rocket_update(); 
}

static void
toggle_fire_width(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = (val + NUM_FIRE_WIDTHS + dir) % NUM_FIRE_WIDTHS;
	(*(unsigned char *) m->value) = (unsigned char) val; 
	m->svalue = (char *) battleship_firewidth_s[val];
	battleship_rocket_update(); 
}

static void
toggle_fire_colour(struct battleship_menu *m, char dir)
{
	int val = (int) (*(unsigned char *) m->value);
	val = (val + NUM_PSYSTEM_COLOURS + dir) % NUM_PSYSTEM_COLOURS;
	(*(unsigned char *) m->value) = (unsigned char) val; 
	m->svalue = (char *) psystem_colours_s[val];
	battleship_rocket_update(); 
}
