/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file battleship_menu.h 
 *
 *  Handels menus.
 *
 *  These files are responsible for the game logic related to menus. They
 *  take care of the movement in and between menus as well as toggling
 *  some variables connect to the menu.
 *  
 */
#ifndef BATTLEHIP_MENU_H
#define BATTLEHIP_MENU_H

/** Structure describing a menu.  */
struct battleship_menu
{
	/** The name of this menu */
	char *name;
	/** The menus parent, only set when menu is activated
	 *  from a parent menu
	 */
	struct battleship_menu *parent;
	/** The submenus or menutimes for this menu */
	struct battleship_menu **item;
	/** The number of submenus or menutimes for this menu */
	unsigned char size;
	/** String representing the current value of this menu item */
	char *svalue;
	/** Value controlled by this menu item */
	void *value;
	/** Function to call to alter the menus value when it is toggled */
	void (*action)(struct battleship_menu *, char);
};

/** Inititlizes the game menus */
void
battleship_menu_init();

/** Called when the user pushes up in a menu 
 *  
 *  Move up one menu item.
 */
void 
battleship_menu_up();

/** Called when the user pushes down in a menu 
 *
 *  Moves down one menu item.
 */
void 
battleship_menu_down();

/** Called when the user pushes left in a menu 
 *
 *  This toggles (or decreases) the value connected to the current menu
 *  item.
 */
void 
battleship_menu_left();

/** Called when the user pushes right in a menu 
 *
 *  This toggles (or increases) the value connected to the current menu
 *  item.
 */
void 
battleship_menu_right();


/** Called when the user pushes enter 
 *
 *  This toggles (or increases) the value connected to the current menu.
 *  Or if the current menu item has no value the current menu becomes
 *  the current selected item.
 */
void 
battleship_menu_toggle();

/** Called when the user pushes esc 
 *
 *  Cancles the current menu. Goes to the parent menu.
 */
void 
battleship_menu_cancel();

/** Sets the given menu to the current menu and initiliez it
 *  @param menu The new menu
 */
void
battleship_menu_load(struct battleship_menu *menu);


/** The main menu, used at start of game. */
struct battleship_menu main_menu;
/** The options menu, used in game. */
struct battleship_menu options_menu;

/** The current menu */
struct battleship_menu *curr_menu;
/** The currently selected item in the current menu */
unsigned char curr_menu_sel;

#endif /* BATTLEHIP_MENU_H */
