/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file battleship_view.c 
 *
 *  These files are responsible for the view part of the program.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "battleship_view.h"
#include "battleship_model.h"
#include "battleship_menu.h"
#include "battleship_rocket.h"
#include "glfont.h"
#include "glprimitive.h"
#include "perlin.h"
#include "lsystem.h"

/** Increaseing this value will move the two players boards apart */
#define BOARD_GAP 0

/** How far up on the board to draw the vertical grid */
#define GRID_TOP_Y 2.0  	// - For the normal on the diagonl stuff. This 
/** How far out fromt he origin to draw the vertical gird */
#define GRID_TOP_Z 0.5
/** How far up on the board to draw the horizontal grid */
#define GRID_BOTTOM_Y -1.5 
/** How far out fromt he origin to draw the horizonal gird */
#define GRID_BOTTOM_Z 2.0  	// - And this should be equal

/** The depth of grid blocks */
#define BLOCK_DEPTH 0.8
/** The heigh/width of grid blocks */
#define BLOCK_DIM 0.9
/** Grid blocks are drawn with some space around them, this value is how
 *  height/wide they are including that space.
 */
#define BLOCK_REAL_DIM 1.0

/** Width of the overhang at the top of the game */
#define BORDER_TOP_WIDTH 1.5
/** Amount of space between top of grid and top of game */
#define BORDER_TOP_GAP 0.5
/** Amount of space between front of horizontal grid and front of game */
#define BORDER_BOTTOM_GAP 0
/** Width of the cubes that make up the border */
#define BORDER_WIDTH 0.2

/** The width of the slots at the side of the board */
#define SLOT_WIDTH 4
/** The height (along z) of thelarger slot on the right side of the board */
#define SLOT_HEIGHT 6

/** Length of wide part of a peg */
#define PEG_LEN_1 BLOCK_DEPTH 
/** Length of the skinny part of a peg */
#define PEG_LEN_2 (BLOCK_DEPTH - 0.2)
/** Diameter of the wide part of a peg */
#define PEG_DIAM_1 0.3
/** Diameter of the skinny part of a peg */
#define PEG_DIAM_2 0.15

/** The height of a ships Hull */
#define SHIP_HULL_HEIGHT 0.4
/** The diameter of the ships hull */
#define SHIP_HULL_DIAM 0.15
/** The height of a ships deck */
#define SHIP_DECK_HEIGHT 0.2

/** The left of the screen, for writting font */
#define FONT_X0 -16
/** The right of the screen, for writting font */
#define FONT_X1 16
/** The top of the screen, for writting font */
#define FONT_Y0 11
/** The bottom of the screen, for writting font */
#define FONT_Y1 -12

/** The width of the picture frame */
#define PICTURE_FRAME_R 0.5
/** The width/height of the picture */
#define PICTURE_DIM 4

/** The width of the table the game board sits on */
#define TABLE_WIDTH 30
/** The height of the table the game board sits on */
#define TABLE_HEIGHT 1.0 
/** The depth of the table the game board sits on */
#define TABLE_DEPTH 26 
/** The top o the table */
#define TABLE_TOP (-(BLOCK_DEPTH/2.0+BORDER_WIDTH) + GRID_BOTTOM_Y)

/** The size of the room width/depth the game is being played in */
#define ROOM_DIM 60 

/** The size of the room height the game is being played in */
#define ROOM_HEIGHT 55

/** The height of the fog cube */
#define FOG_HEIGHT (PEG_LEN_2)

/** X Location of light used to cast shadows */
#define LIGHT_X 0 
/** Y Location of light used to cast shadows */
#define LIGHT_Y (ROOM_HEIGHT/2.0)
/** Z Location of light used to cast shadows */
#define LIGHT_Z 0

/** The current width of the display */
static GLint width;
/** The current height of the display */
static GLint height;

/** Translations of game board by user */
GLfloat userTranslate[3];
/** Rotation of game board by user */
GLfloat userRotate[3];
/** Translations of menu by user */
GLfloat menuTranslate[3];
/** Rotation of menu by user */
GLfloat menuRotate[3];
/** Translation of the game board done by game logic */
GLfloat gameTranslate[3];
/** Rotation of the game board done by game logic */
GLfloat gameRotate[3];
/** Stores shadow projection matracies */
GLfloat shadow_matrix[NUM_SHADOWS - 1][16];

/** Sets up the default perspective, translation and lighting */
static void set_perspective();

/** Creates a shadow projection matrix 
 *  @param light A point (x, y, z, 1) which is the location of the light source
 *  @param p0 A point (x, y, z, 1) on the plane the shadows will be projected on
 *  @param pn A vector (x, y, z, 0) which is the normal to the plane
 *  @param m A 16 (4x4) element matrix in which the shadow matrix will
 *  		 be returned
 */
static void createShadowMatrix(GLfloat *light, GLfloat *p0, GLfloat *pn,
								GLfloat *m);

/** Draws shadows */
static void drawShadows();

/** This is the main drawing function. It draws the game */
static void drawGame();

/** Draws A grid of boxes 
 *  @return The list id to call in order to draw this item.
 **/
static GLuint drawGrid();
/** Draws a box inside the currently selected gird location */
static void drawGridSelect(unsigned char pnum);
/** Draws the black border that makes up the game
 *  @return The list id to call in order to draw this item.
 */
static GLuint drawBorder();
/** Draws a peg/
 *  @return The list id to call in order to draw this item.
 */
static GLuint drawPeg();

/** Diffuse colour used for white peg */
static const GLfloat diff_w[] = {1.0, 1.0, 1.0, 1.0};
/** Specular colour used for white peg */
static const GLfloat spec_w[] = {0.1, 0.1, 0.1, 1.0};
/** Shinnyness used for white peg */
static const GLfloat shinny_w = 1;
/** Diffuse colour used for red peg */
static const GLfloat diff_r[] = {1.0, 0.0, 0.0, 1.0};
/** Specular colour used for red peg */
static const GLfloat spec_r[] = {0.1, 0.1, 0.1, 1.0};
/** Shinnyness used for red peg */
static const GLfloat shinny_r = 1;

/** Draws pegs on the board */
static void drawPegs(unsigned char pnum);
/** Draws pegs in the slots on the side of the game */
static void drawExtraPegs(unsigned char pnum);

/** Diffuse colour used for ships */
static const GLfloat diff_ship[] = {0.5, 0.5, 0.5, 1.0};
/** Specular used for ships */
static const GLfloat spec_ship[] = {0.1, 0.1, 0.1, 1.0};
/** Shinnyness used for ships */
static const GLfloat shinny_ship = 1;

/** Draws a carrier 
 *  @return The list id to call in order to draw this item
 */
static GLuint drawCarrier();

/** Draws a battleship 
 *  @return The list id to call in order to draw this item
 */
static GLuint drawBattleship();

/** Draws a destroyer
 *  @return The list id to call in order to draw this item
 */
static GLuint drawDestroyer();

/** Draws a sub
 *  @return The list id to call in order to draw this item
 */
static GLuint drawSub();

/** Draws a PT boat
 *  @return The list id to call in order to draw this item
 */
static GLuint drawPT();

/** Draws a ship
 *  @param snum The ship to draw
 *  @return The list id to call in order to draw this item.
 */
static GLuint drawShip(enum battleship_shiptype snum);

/** Draws all of a players ships.
 *  @param pnum The player
 */
static void drawShips(unsigned char pnum);

/** Draws fog obscuring a players ships.  */
static void drawFog();

/** Draws the current menu */
static void drawMenu();

/** Draws the letters of the font allowing a player to select their name */
static void drawNameSelector();

/** Draws game state info */
static void drawStatus();

/** Draws the glprimitive_clock */
static void drawClock(bool test);

/** Draws the glprimitive_mug with texture mapped coffie */
static void drawMug(bool test);

/** Draws picture and frame */
static void drawPicture(unsigned char pnum);

/** Draws the table the game sits on */
static void drawTable();

/** Draws the ceiling */
static void drawCeiling();

/** Draws the floor */
static void drawFloor();

/** Draws the walls, using l-system patter */
static void drawWall();

/** Draws the rocket */
static void drawRocket();

/** The number of points in a rocket path */
#define PATH_SIZE 5 
/** The rocket paths used by the game 
 *  Note thate the first path in this array (which would corrispond to
 *  ROCKET_OFF is used by the rocket menu
 */
static GLdouble rocket_path[NUM_ROCKET_TYPES][PATH_SIZE*3 + 1] =
{
	/*
	{
	5,
	0.0, -5.0, 0.0, 
	0.0, 0.0, -5.0, 
	0.0, 5.0, 0.0,
	0.0, 0.0, 5.0,
	0.0, -5.0, 0.0
	},
	*/
	/*
	{
	5,
	0.0, -5.0, 0.0, 
	5.0, 0.0, 0.0, 
	0.0, 5.0, 0.0,
	-5.0, 0.0, 0.0,
	0.0, -5.0, 0.0
	},
	*/
	/*	
	{
	5,
	-5.0, 0.0, 0.0, 
	0.0, 0.0, 5.0, 
	5.0, 0.0, 0.0,
	0.0, 0.0, -5.0,
	-5.0, 0.0, 0.0,
	0.0, 0.0, -5.0, 
	},
	*/
	{
		PATH_SIZE,  
		6.0, -10, 0.0, 
		3.0, -4, 0.0,
		3.0, 0, 0.0,
		3.0, 4, 0.0,
		6.0, 10, 0.0,
	},
	{
		PATH_SIZE,  
		0, TABLE_TOP - 2, GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM, 
		0, 14, GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM*0.5,
		0, 15, 0.0, 
		0, 14, -(GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM*0.5),
		0, TABLE_TOP + BLOCK_DEPTH, 0
	},
	{
		PATH_SIZE, 
		0, TABLE_TOP - 2, GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM, 
		10, 8, GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM*0.5,
		10, 8, 0,
		10, 8, -(GRID_BOTTOM_Z + GRID_DIM*BLOCK_REAL_DIM + 2),
		0, TABLE_TOP + BLOCK_DEPTH, 0
	},
};


#ifndef TEXTURE_DIR
	/** The directory containg textures */
	#define TEXTURE_DIR "../data/"
#endif
/** The number of textures used */
#define NUM_TEXTURES 9
/** Array which stores texture ids */
static GLuint texture_id[NUM_TEXTURES];
/** Location of textures */
static const char *texture_file[NUM_TEXTURES] =
{
	TEXTURE_DIR "table.rgb",
	TEXTURE_DIR "table_leg.rgb",
	TEXTURE_DIR "coffie.rgb",
	TEXTURE_DIR "ceiling.rgb",
	TEXTURE_DIR "floor.rgb",
	TEXTURE_DIR "pic1.rgb",
	TEXTURE_DIR "pic2.rgb",
	TEXTURE_DIR "left.rgb",
	TEXTURE_DIR "right.rgb",
};
/** Texture sizes */
static const int texture_dim[NUM_TEXTURES][2] =
{
	{256, 256},
	{64, 64},
	{64, 64},
	{64, 64},
	{64, 64},
	{256, 256},
	{256, 256},
	{128, 512},
	{128, 512},
};
/** Used to index into texture array */
enum texture_index
{
	TEXTURE_TABLE = 0,
	TEXTURE_TABLE_LEG,
	TEXTURE_COFFIE,
	TEXTURE_CEILING,
	TEXTURE_FLOOR,
	TEXTURE_PIC1,
	TEXTURE_PIC2,
	TEXTURE_LEFT,
	TEXTURE_RIGHT,
};
/** Texture id for fog texture */
static GLuint fog_texture;
/** Texture id for lsystem texture */
static GLuint lsys_texture;

/** Loads a texture from a file into memory */
static GLuint loadTextureFile(char *filename, int w, int h);

/** Loads a texture form perexisting data */
static GLuint loadTextureData(void *data, int w, int h, bool alpha);

/** Lsystem texture */
static void loadLsystem();
	
/** Creates the fog texture */
static void loadFog();

/** A bunch of functions use a display list to draw things.
 *  They work by drawing the thing once. Then returning the display
 *  is. This marcro creats the top part of the function.
 */
#define START_LIST 						\
	static GLuint list = 0; 			\
	if (list == 0) 						\
	{									\
		list = glGenLists(1);			\
		glNewList(list, GL_COMPILE);	\
		glPushMatrix();					\

/** A bunch of functions use a display list to draw things.
 *  They work by drawing the thing once. Then returning the display
 *  is. This marcro creats the end part of the function.
 */
#define END_LIST						\
		glPopMatrix();					\
		glEndList();					\
	}									\
	return list;						


void
battleship_view_init()
{
	static bool first_time = true;

	menuTranslate[0] = 0;
	menuTranslate[1] = 0;
	menuTranslate[2] = 0;
	menuRotate[0] = 0;
	menuRotate[1] = 0;
	menuRotate[2] = 0;
	userTranslate[0] = 0;
	userTranslate[1] = 0;
	userTranslate[2] = 0;
	userRotate[0] = 0;
	userRotate[1] = 0;
	userRotate[2] = 0;
	gameTranslate[0] = 0;
	gameTranslate[1] = 0;
	gameTranslate[2] = 0;
	gameRotate[0] = 0;
	gameRotate[1] = 0;
	gameRotate[2] = 0;

	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glShadeModel(GL_SMOOTH);

	if (first_time)
	{
		first_time = false;
		battleship_rocket_init();
		battleship_rocket_set_detail(0.1);
		battleship_view_ready_rocket();

		//load textures
		size_t i;
		for (i = 0; i < NUM_TEXTURES; i++)
		{
			texture_id[i] = 
				loadTextureFile((char *)texture_file[i], 
							texture_dim[i][0],
							texture_dim[i][1]
							);
		}
		glBindTexture(GL_TEXTURE_2D, 0);
		fog_texture = 0;
		lsys_texture = 0;
		
		//Position of the light
		GLfloat light[4] = {LIGHT_X, LIGHT_Y, LIGHT_Z, 1.0};
		
		//Table top
		GLfloat plane_normal[4] = {0.0, 1.0, 0.0, 0.0};
		GLfloat plane_point[4] = {0.0, TABLE_TOP, 0.0, 1.0};
		createShadowMatrix(light, plane_point, plane_normal, shadow_matrix[0]);
		//Back wall
		GLfloat plane_normal1[4] = {0.0, 0.0, -1.0, 0.0};
		GLfloat plane_point1[4] = {0.0, 0.0, ROOM_DIM/2.0, 1.0};
		createShadowMatrix(light, plane_point1, plane_normal1,shadow_matrix[1]);
		//Front wall
		GLfloat plane_normal2[4] = {0.0, 0.0, 1.0, 0.0};
		GLfloat plane_point2[4] = {0.0, 0.0, -ROOM_DIM/2.0, 1.0};
		createShadowMatrix(light, plane_point2, plane_normal2,shadow_matrix[2]);
		//Shadows on floor
		GLfloat plane_normal3[4] = {0.0, 1.0, 0.0, 0.0};
		GLfloat plane_point3[4] = {0.0, -ROOM_HEIGHT/2.0, 0.0, 1.0};
		createShadowMatrix(light, plane_point3, plane_normal3,shadow_matrix[3]);
	}
}

void 
battleship_view_getsize(int *w, int *h)
{
	*w = width;
	*h = height;
}

void 
battleship_view_resize(int w, int h)
{
	width = w;
	height = h;
	
	battleship_view_refresh();
}

static void
set_perspective()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, width, height);
	gluPerspective(30.0, (GLdouble) width/(GLdouble) height, 30.0, 500.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	if (curr_menu)
		glClearColor(0.0, 0.0, 0.4, 0.0);
	else
		glClearColor(0.0, 0.0, 0.0, 0.0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (do_textures)
		glEnable(GL_TEXTURE_2D);
	else
		glDisable(GL_TEXTURE_2D);
	
	GLfloat light_pos[] = {0.0, 10.0, 0.0, 1.0};
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glEnable(GL_LIGHT0);
	
	glTranslatef(0, 0, -50);
}

void
battleship_view_main()
{
	set_perspective();

	//Translation !
	if (!curr_menu)
	{
		//Translation done by game
		if (do_test == TEST_NONE)
		{
			//Draws game state info on top
			drawStatus();
			
			glTranslatef(gameTranslate[0], 
					gameTranslate[1] - 4, 
					gameTranslate[2]);
			glRotatef(gameRotate[0], 1, 0, 0);
			glRotatef(gameRotate[1], 0, 1, 0);
			glRotatef(gameRotate[2], 0, 0, 1);
		}
		
		//user rotation
		glRotatef(userRotate[0], 1, 0, 0);
		glRotatef(userRotate[1], 0, 1, 0);
		glRotatef(userRotate[2], 0, 0, 1);
		glTranslatef(userTranslate[0], userTranslate[1], userTranslate[2]);
	}
	else
	{
		//user rotation of menu
		glRotatef(menuRotate[0], 1, 0, 0);
		glRotatef(menuRotate[1], 0, 1, 0);
		glRotatef(menuRotate[2], 0, 0, 1);
		glTranslatef(menuTranslate[0], menuTranslate[1], menuTranslate[2]);
	}

	/*
	glBegin(GL_LINES);
		glVertex3f(0.0, 5.0, 0.0);
		glVertex3f(0.0, -5.0, 0.0);
		glVertex3f(-5.0, 0.0, 0.0);
		glVertex3f(5.0, 0.0, 0.0);
	glEnd();
	*/

	switch (do_test)
	{
		case TEST_PRIMITIVE:
			glprimitive_test(4.0, 15);
		break;
		case TEST_CLOCK:
			drawClock(true);
		break;
		case TEST_MUG:
			drawMug(true);
		break;
		case TEST_PEG:
			glEnable(GL_NORMALIZE);
			glScalef(4.0, 4.0, 4.0);
			glCallList(drawPeg());
			glDisable(GL_NORMALIZE);
		break;	
		case TEST_SHIPS:
			glEnable(GL_NORMALIZE);
			glScalef(2.0, 2.0, 2.0);
			glRotatef(30, 1, 0, 0);
			glTranslatef(0.0, 2, -3);
			drawShips(0);
			glTranslatef(0.0, GRID_BOTTOM_Y, -GRID_TOP_Z + GRID_BOTTOM_Z);
			glRotatef(90, 1, 0, 0);
			glRotatef(180, 0, 1, 0);
			glCallList(drawGrid());
			glDisable(GL_NORMALIZE);
		break;	
		case TEST_FONT:
			glTranslatef(-14, 8, 0);
			glfont_test(3, 0, 0);
		break;
		case TEST_FOG:
			drawMenu();
			glTranslatef(0.0, -5.0, 15.0);
			glCallList(drawGrid());
			drawFog();
		break;
		case TEST_LSYS:
			drawMenu();
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 0.0, 40);
			drawWall();
		break;
		case TEST_ROCKET:
			drawRocket();
			static int count = 0;
			++count;
			if (count > 10)
			{
				count = 0;
				if (!battleship_rocket_move())
				{
					game_rocket.type = 
							(game_rocket.type + 1) % NUM_ROCKET_TYPES;
					battleship_view_ready_rocket();
				}
			}
		break;
		case TEST_NONE:
			if (curr_menu)
			{
				if (name_selector.enabled)
					drawNameSelector();
				else
				{
					drawMenu();
					if (!strcmp(curr_menu->name, "Fog"))
					{
						drawFog();
					}
					else if (!strcmp(curr_menu->name, "L-Systems"))
					{
						glRotatef(180, 0.0, 1.0, 0.0);
						glTranslatef(0.0, 30.0, 200);
						drawWall();
					}
					else if (!strcmp(curr_menu->name, "Rocket") ||
							!strcmp(curr_menu->name, "Fire"))
					{
						glTranslatef(7.0, 0.0, 0);
						if (battleship_rocket_get_path() != rocket_path[0])
							battleship_rocket_set_path(rocket_path[0]);
						//glRotatef(-90, 0.0, 1.0, 0.0);
						drawRocket();
						if (!battleship_rocket_move())
							battleship_rocket_reset();
					}
				}
			}
			else
			{
				if (do_shadows[SHADOW_ALL])
					drawShadows();
				drawGame();
			}
		break;
		default:
			//DPRINT("Unknown test type %i\n", do_test);
		break;
	}

	battleship_view_swap_buffers();
}

void battleship_view_set_translate(int axis, float amount, unsigned char type)
{
	if (axis < 0 || axis > 2)
	{
		//DPRINT("Invalid axis to set translate.\n");
		return;
	}
	
	switch (type)
	{
		case 'u' : 
			if (curr_menu)
				menuTranslate[axis] = amount; 
			else
				userTranslate[axis] = amount; 
		break;
		case 'U' :
			if (curr_menu)
				menuTranslate[axis] += amount; 
			else
				userTranslate[axis] += amount; 
		break;	
		case 'g' : gameTranslate[axis] = amount; break;
		case 'G' : gameTranslate[axis] += amount; break;
		default:
			//DPRINT("Unknow type to set translate.\n");
		break;
	}
}

void battleship_view_set_rotate(int axis, float amount, unsigned char type)
{
	if (axis < 0 || axis > 2)
	{
		//DPRINT("Invalid axis to set rotate.\n");
		return;
	}

	switch (type)
	{
		case 'u' : 
			if (curr_menu)
				menuRotate[axis] = amount;
			else
				userRotate[axis] = amount; 
		break;
		case 'U' :
			if (curr_menu)
				menuRotate[axis] += amount;
			else
				userRotate[axis] += amount; 
		break;
		case 'g' : gameRotate[axis] = amount; break;
		case 'G' : gameRotate[axis] += amount; break;
		default:
			//DPRINT("Unknow type to set rotate.\n");
		break;
	}
}

float battleship_view_get_translate(int axis, unsigned char type)
{
	if (axis < 0 || axis > 2)
	{
		//DPRINT("Invalid axis to get translate.\n");
		return 0.0;
	}

	float r = 0.0;
	switch (type)
	{
		case 'u' : 
			if (curr_menu)
				r = menuTranslate[axis]; 
			else 
				r = userTranslate[axis]; 
		break;
		case 'g' : r = gameTranslate[axis]; break;
		default:
			//DPRINT("Unknow type to get translate.\n");
		break;
	}

	return r;
}

float battleship_view_get_rotate(int axis, unsigned char type)
{
	if (axis < 0 || axis > 2)
	{
		//DPRINT("Invalid axis to get rotate.\n");
		return 0.0;
	}

	float r = 0.0;
	switch (type)
	{
		case 'u' : 
			if (curr_menu)
				r = menuRotate[axis];
			else
				r = userRotate[axis];
		break;
		case 'g' : r = gameRotate[axis]; break;
		default:
			//DPRINT("Unknow type to get rotate.\n");
		break;
	}

	return r;
}

static void createShadowMatrix(GLfloat *light, GLfloat *p0, GLfloat *pn, 
								GLfloat *m)
{
	//These are the coefficients corrisponding to the plane 
	//equation ax + by + cz + d = 0
	GLfloat plane[4] = {pn[0], pn[1], pn[2], 
					 -(p0[0]*pn[0] + p0[1]*pn[1] + p0[2]*pn[2])
					};

	GLfloat dot = plane[0] * light[0] + 
		plane[1] * light[1] + 
		plane[2] * light[2] + 
		plane[3] * light[3];

	m[0] = dot - light[0] * plane[0];
	m[4] = 0.0 - light[0] * plane[1];
	m[8] = 0.0 - light[0] * plane[2];
	m[12] = 0.0 - light[0] * plane[3];

	m[1] = 0.0 - light[1] * plane[0];
	m[5] = dot - light[1] * plane[1];
	m[9] = 0.0 - light[1] * plane[2];
	m[13] = 0.0 - light[1] * plane[3];

	m[2] = 0.0 - light[2] * plane[0];
	m[6] = 0.0 - light[2] * plane[1];
	m[10] = dot - light[2] * plane[2];
	m[14] = 0.0 - light[2] * plane[3];

	m[3] = 0.0 - light[3] * plane[0];
	m[7] = 0.0 - light[3] * plane[1];
	m[11] = 0.0 - light[3] * plane[2];
	m[15] = dot - light[3] * plane[3];
}

static void drawShadows()
{
	glEnable(GL_STENCIL_TEST);
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	
	//Shadows on table
	if (do_shadows[SHADOW_TABLE])
	{
		glStencilFunc(GL_ALWAYS, 1, ~0);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		//This is a sort of hack. Just draw the top of the table
		//in the same fashion that glprimivtive_box would do. So
		//shadows don't bleed onto other parts
		GLdouble x = -TABLE_WIDTH/2.0;
		GLdouble y = -TABLE_HEIGHT + TABLE_TOP;
		GLdouble z = -TABLE_DEPTH/2.0;
		GLdouble w = TABLE_WIDTH;
		GLdouble h = TABLE_HEIGHT;
		GLdouble d =  TABLE_DEPTH;
		GLdouble s = 1.0/(h + w);
		GLdouble t = 1.0/(d + h);
		glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_TABLE]);
		glBegin(GL_QUADS);
			glNormal3f(0, 1, 0);
			glTexCoord2f(h*s, 0); glVertex3f(x, y + h, z);
			glTexCoord2f(h*s, d*t); glVertex3f(x, y + h, z + d);
			glTexCoord2f(1, d*t); glVertex3f(x + w, y + h, z +d);
			glTexCoord2f(1, 0); glVertex3f(x + w, y + h, z);
		glEnd();	
		glBindTexture(GL_TEXTURE_2D, 0);
		//drawTable();
	}
	
	//Shadows on back wall
	if (do_shadows[SHADOW_BACK_WALL])
	{
		glStencilFunc(GL_ALWAYS, 2, ~0);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		drawWall();
	}
	
	//Shadows on front wall
	if (do_shadows[SHADOW_FRONT_WALL])
	{
		glStencilFunc(GL_ALWAYS, 3, ~0);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		glPushMatrix();
			glRotatef(180, 0.0, 1.0, 0.0);
			drawWall();
		glPopMatrix();	
	}

	//Shadows on floor
	if (do_shadows[SHADOW_FLOOR])
	{
		glStencilFunc(GL_ALWAYS, 4, ~0);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		drawFloor();
	}

	//Do the shadows
	glPushMatrix();
	glPushAttrib(GL_ENABLE_BIT);
		glColor4f(0.0, 0.0, 0.0, 0.5);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);
		
		//Shadows on table
		if (do_shadows[SHADOW_TABLE])
		{
			glStencilFunc(GL_EQUAL, 1, ~0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
			glPushMatrix();
				glMultMatrixf(shadow_matrix[0]);
				drawClock(false);
				drawMug(false);
			glPopMatrix();
		}

		//Shadows on front wall
		if (do_shadows[SHADOW_BACK_WALL])
		{
			glStencilFunc(GL_EQUAL, 2, ~0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
			glPushMatrix();
				glMultMatrixf(shadow_matrix[1]);
				drawPicture(0);
			glPopMatrix();	
		}

		//Shadows on back wall
		if (do_shadows[SHADOW_FRONT_WALL])
		{
			glStencilFunc(GL_EQUAL, 3, ~0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
			glPushMatrix();
				glMultMatrixf(shadow_matrix[2]);
				glRotatef(180, 0.0, 1.0, 0.0);
				drawPicture(1);
			glPopMatrix();	
		}

		//Shadows on floor
		if (do_shadows[SHADOW_FLOOR])
		{
			glStencilFunc(GL_EQUAL, 4, ~0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
			glPushMatrix();
				glMultMatrixf(shadow_matrix[3]);
				drawTable();
			glPopMatrix();	
		}
	glPopAttrib();
	glPopMatrix();
	
	glDisable(GL_STENCIL_TEST);
}

static void drawGame()
{
	//Draw some texture mappped things
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	drawCeiling();
	drawFloor();
	//Draw walls
	glPushMatrix();
		drawWall();
		glRotatef(90, 0.0, 1.0, 0.0);
		drawWall();
		glRotatef(90, 0.0, 1.0, 0.0);
		drawWall();
		glRotatef(90, 0.0, 1.0, 0.0);
		drawWall();
	glPopMatrix();

	drawTable();
	drawClock(false);
	drawMug(false);

	GLuint g = drawGrid();
	GLuint tb = drawBorder(); 
	size_t p;
	for (p = 0; p < 2; p++)
	{
		glTranslatef(0.0, 0.0, BOARD_GAP);

		//Draw Pegs
		drawPegs(p);
		drawExtraPegs(p);

		//Draw Border
		glCallList(tb);

		drawShips(p);

		//Draw Gird
		glPushMatrix();
			glTranslatef(0.0, GRID_TOP_Y, GRID_TOP_Z);
			if (game_state == GAME_PLAYING || 
					game_state == GAME_AI_PLAYING)
			{
				drawGridSelect(p);
			}
			glCallList(g);

			glTranslatef(0.0, -GRID_TOP_Y + GRID_BOTTOM_Y, 
					-GRID_TOP_Z + GRID_BOTTOM_Z);
			glRotatef(90, 1, 0, 0);
			glRotatef(180, 0, 1, 0);
			glCallList(g);

			//fog
			if (player[p].fog && 
					!(game_state == GAME_PLACE_SHIPS && p == curr_player))
			{
				drawFog();
			}
		glPopMatrix();

		//draw pictures
		drawPicture(p);

		if (p == curr_player && game_state == GAME_FIREING)
			drawRocket();
		
		glRotatef(180, 0, 1, 0);
	}
	
}

static GLuint
drawGrid()
{

	static const GLfloat diff_water[] = {0.0, 0.2, 0.9, 0.4};
	static const GLfloat spec_water[] = {0.0, 0.0, 0.0, 0.4};
	static const GLfloat shinny_water = 1;

	START_LIST
			glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_water);
		    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_water);
			glMaterialf(GL_FRONT, GL_SHININESS, shinny_water);

			glTranslatef(-GRID_DIM/2.0, 0.0, -BLOCK_DEPTH/2.0);
			size_t x, y;
			for (x = 0; x < GRID_DIM; x++)
			{
				glPushMatrix();
				for (y = 0; y < GRID_DIM; y++)
				{
					//Note i don't use my primitive here because I don't
					//need the back face. Its a good way to remove 400
					//faces
					glBegin(GL_QUADS);
						//Front
						glNormal3f(0, 0, 1);
						glVertex3f(0, 0, 0 + BLOCK_DEPTH);
						glVertex3f(0 + BLOCK_DIM, 0, BLOCK_DEPTH);
						glVertex3f(0 + BLOCK_DIM, BLOCK_DIM, BLOCK_DEPTH);
						glVertex3f(0, 0 + BLOCK_DIM, 0 + BLOCK_DEPTH);
						//Left
						if (x == 0)
						{
							glNormal3f(-1, 0, 0);
							glVertex3f(0, 0, 0);
							glVertex3f(0, 0, BLOCK_DEPTH);
							glVertex3f(0, BLOCK_DIM, 0 + BLOCK_DEPTH);
							glVertex3f(0, BLOCK_DIM, 0);
						}
						//Rigt
						if (x == GRID_DIM - 1)
						{
							glNormal3f(1, 0, 0);
							glVertex3f(BLOCK_DIM, 0, BLOCK_DEPTH);
							glVertex3f(BLOCK_DIM, 0, 0);
							glVertex3f(BLOCK_DIM, BLOCK_DIM, 0);
							glVertex3f(BLOCK_DIM, BLOCK_DIM, 0 + BLOCK_DEPTH);
						}
						//Top
						if (y == GRID_DIM - 1)
						{
							glNormal3f(0, 1, 0);
							glVertex3f(0, BLOCK_DIM, 0);
							glVertex3f(0, BLOCK_DIM, BLOCK_DEPTH);
							glVertex3f(BLOCK_DIM, BLOCK_DIM, BLOCK_DEPTH);
							glVertex3f(BLOCK_DIM, BLOCK_DIM, 0);
						}
						//Bottom
						if (y == 0)
						{
							glNormal3f(0, -1, 0);
							glVertex3f(0, 0, 0);
							glVertex3f(0, 0, BLOCK_DEPTH);
							glVertex3f(BLOCK_DIM, 0, BLOCK_DEPTH);
							glVertex3f(BLOCK_DIM, 0, 0);
						}
					glEnd();
					glTranslatef(0.0, BLOCK_REAL_DIM, 0.0);
				}
				glPopMatrix();
				glTranslatef(BLOCK_REAL_DIM, 0.0, 0.0);
			}
	END_LIST
}

static void 
drawGridSelect(unsigned char pnum)
{
	static const GLfloat diff[] = {0.0, 0.2, 0.9, 1.0};
	static const GLfloat spec[] = {0.0, 0.0, 0.0, 1.0};
	static const GLfloat shinny = 1;
	
	glPushMatrix();
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny);

		glTranslatef(
		  -GRID_DIM/2.0 + BLOCK_REAL_DIM * player[pnum].sel_x,
		  BLOCK_REAL_DIM * (GRID_DIM - 1)-BLOCK_REAL_DIM * player[pnum].sel_y, 
		  -BLOCK_DEPTH/2.0);
		glprimitive_box(0, 0, 0, BLOCK_DIM, BLOCK_DIM, BLOCK_DEPTH);
	glPopMatrix();
}
static GLuint
drawBorder()
{
	static const GLfloat diff[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat spec[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat shinny = 1;
	
	START_LIST
		static const GLfloat G = BORDER_WIDTH;
		GLfloat left = -GRID_DIM/2.0 * BLOCK_REAL_DIM - SLOT_WIDTH + G;
		GLfloat bottom = GRID_BOTTOM_Y - BLOCK_DEPTH/2.0 - G;
		GLfloat top = GRID_TOP_Y + GRID_DIM * BLOCK_REAL_DIM + BORDER_TOP_GAP + G;

		//Top
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny);
		//Left Side
		glprimitive_box(left, bottom, 0, 
				 G, top - bottom, BORDER_TOP_WIDTH);
		//Right Side
		glprimitive_box(-left - G, bottom, 0,
				 G, top - bottom, BORDER_TOP_WIDTH);
		//Top
		glprimitive_box(left, top - G, 0,
				 -2*left, G, BORDER_TOP_WIDTH);
		//Planes in the Middle of top Board (or the back)
		  //char 0 to -G if BOARD_GAP != 0
		glprimitive_box(left, bottom, 0,
				 2*-left, top - bottom, G);
		
		//Bottom
		top = 0;
		GLfloat front = GRID_DIM * BLOCK_REAL_DIM + GRID_BOTTOM_Z + G + BORDER_BOTTOM_GAP;
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny);
		//Left Side
		glprimitive_box(left, bottom, 0,
				 G, top - bottom, front);
		//Right
		glprimitive_box(-left - G, bottom, 0,
				 G, top - bottom, front);
		//Bottom
		glprimitive_box(left, bottom, 0,
				 -2*left, G, front);
	
		top = GRID_BOTTOM_Y + BLOCK_DEPTH/2.0;
		//front
		glprimitive_box(left, bottom, front - G,
				 -2*left, top - bottom, G);
		//Peg Divider Front facing player
		glprimitive_box(-left - SLOT_WIDTH + G, bottom, SLOT_HEIGHT - G,
				 SLOT_WIDTH - G, top - bottom, G);

		//Status thing
		glBegin(GL_QUADS);
			glNormal3f(0, 1, 1);
			glVertex3f(left + G, bottom + G, GRID_BOTTOM_Z);
			glVertex3f(-left - G, bottom + G, GRID_BOTTOM_Z);
			glVertex3f(-left - G, GRID_TOP_Y, G);
			glVertex3f(left + G, GRID_TOP_Y, G);
		glEnd();

		//Right decal
		left = -GRID_DIM/2.0*BLOCK_REAL_DIM - SLOT_WIDTH + BLOCK_REAL_DIM;
		top = GRID_TOP_Y + GRID_DIM*BLOCK_REAL_DIM - 0.1;
		bottom = GRID_TOP_Y + 0.1;
		front = GRID_TOP_Z;
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
		glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_LEFT]);
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0, 0.0);
			glVertex3f(left, top, front);
			glTexCoord2f(0.0, 1.0);
			glVertex3f(left, bottom, front);
			glTexCoord2f(1.0, 1.0);
			glVertex3f(left + 2.5*BLOCK_REAL_DIM, bottom, front);
			glTexCoord2f(1.0, 0.0);
			glVertex3f(left + 2.5*BLOCK_REAL_DIM, top, front);
		glEnd();

		left = GRID_DIM/2.0*BLOCK_REAL_DIM+SLOT_WIDTH/2.0-1.5*BLOCK_REAL_DIM;
		glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_RIGHT]);
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0, 0.0);
			glVertex3f(left, top, front);
			glTexCoord2f(0.0, 1.0);
			glVertex3f(left, bottom, front);
			glTexCoord2f(1.0, 1.0);
			glVertex3f(left + 2.5*BLOCK_REAL_DIM, bottom, front);
			glTexCoord2f(1.0, 0.0);
			glVertex3f(left + 2.5*BLOCK_REAL_DIM, top, front);
		glEnd();
		glBindTexture(GL_TEXTURE_2D, 0);
	END_LIST
}

static GLuint drawPeg()
{
	START_LIST
		/*
		glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);

		glTranslatef(0.0, 0.0, PEG_LEN_2);
		glprimitive_cylinder(PEG_DIAM_1, PEG_LEN_1); 

		glTranslatef(0.0, 0.0, PEG_LEN_1);
		glprimitive_disk(PEG_DIAM_1);

		glTranslatef(0.0, 0.0, -PEG_LEN_1);
		glRotatef(180, 0.0, 1.0, 0.0);
		glprimitive_disk(PEG_DIAM_1);

		glTranslatef(0.0, 0.0, PEG_LEN_2);
		glprimitive_disk(PEG_DIAM_2);
		*/
	   GLUquadricObj *c1 = gluNewQuadric();
       gluQuadricDrawStyle(c1, GLU_FILL);
       GLUquadricObj *c2 = gluNewQuadric();
       gluQuadricDrawStyle(c2, GLU_FILL);
       GLUquadricObj *d1 = gluNewQuadric();
       gluQuadricDrawStyle(d1, GLU_FILL);
       GLUquadricObj *d2 = gluNewQuadric();
       gluQuadricDrawStyle(d2, GLU_FILL);
       GLUquadricObj *d3 = gluNewQuadric();
       gluQuadricDrawStyle(d3, GLU_FILL);

	   gluCylinder(c1,
					   PEG_DIAM_2, PEG_DIAM_2, PEG_LEN_2, 5, 5
				   );

	   glTranslatef(0.0, 0.0, PEG_LEN_2);
	   gluCylinder(c2,
					   PEG_DIAM_1, PEG_DIAM_1, PEG_LEN_1, 8, 8
				   );

	   glTranslatef(0.0, 0.0, PEG_LEN_1);
	   gluDisk(d1, 0, PEG_DIAM_1, 10, 10);

	   glTranslatef(0.0, 0.0, -PEG_LEN_1);
	   glRotatef(180, 0.0, 1.0, 0.0);
	   gluDisk(d2,
					   0, PEG_DIAM_1, 10, 10
				   );

	   glTranslatef(0.0, 0.0, PEG_LEN_2);
	   gluDisk(d3,
					   0, PEG_DIAM_2, 5, 5
				   );
		
		gluDeleteQuadric(c1);
		gluDeleteQuadric(c2);
		gluDeleteQuadric(d1);
		gluDeleteQuadric(d2);
		gluDeleteQuadric(d3);
	END_LIST
}

static void
drawPegs(unsigned char pnum)
{
	
	GLuint pieceList = drawPeg();
	bool white = true;
	pnum = 1 - pnum; //want other players stats

	//set default material
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);

	glPushMatrix();
	//Go to top left
	glTranslatef(-(GRID_DIM/2.0)*BLOCK_REAL_DIM + BLOCK_REAL_DIM/2.0, 
				  GRID_DIM*BLOCK_REAL_DIM + GRID_TOP_Y - BLOCK_REAL_DIM/2.0, 
				  GRID_TOP_Z + BLOCK_DEPTH/2.0 - PEG_LEN_2);
	size_t x, y;
	for (x = 0; x < GRID_DIM; x++)
	{
		glPushMatrix();
		for (y = 0; y < GRID_DIM; y++)
		{
			//check for miss/hit
			bool miss = player[pnum].grid[x][y] == GRID_MISS;
			bool hit = player[pnum].grid[x][y] == GRID_HIT;
		
			//set colour
			if (miss && !white)
			{
				glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
			    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
				glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
				white = true;
			}
			else if (hit && white)
			{
				glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
			    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
				glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
				white = false;
			}
	
			//draw peice
			if (miss || hit)
				glCallList(pieceList);

			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
		}
		glPopMatrix();
		glTranslatef(BLOCK_REAL_DIM, 0.0, 0.0);
	}
	glPopMatrix();
}

static void
drawExtraPegs(unsigned char pnum)
{
	//desribes location of red pegs, each location is reltitive to the privious
	static const GLfloat red[MAX_HITS][6] =
	{
		{((GRID_DIM+1)/2.0)*BLOCK_REAL_DIM, GRID_BOTTOM_Y, GRID_BOTTOM_Z, 0, 0, 0},
		{0, 0, PEG_DIAM_1, 0, 90, 0},
		{-0.5, 0, 0.5, 0, -45, 0},
		{-0.5, 0, PEG_LEN_1 + PEG_LEN_2, 0, -45, 0},
		{-0.5, 0, 0, 0, 0, 0},
		{-0.5, 0, -0.5, 0, 0, 0},
		{0.6, PEG_DIAM_2, 0, 0, -45, 0},
		{0, 0, 0, 0, -100, 0},
		{-2.2, -2*PEG_DIAM_2, -0.6, 0, 55, 0},
		{1.0, 0, 0.0, 0, 0, 0},
		{0.5, 0, 0, 0, 45, 0},
		{-0.5, 0.5, 0, 15, 0, 0},
		{-0.8, -0.2, 0.2, 0, -160, 0},
		{0.5, 0.5, -1, 15, 250, 0},
		{0.5, 0.2, -0.2, 0, 120, 0},
		{0.4, 0, -0.4, 0, 185, 0},
		{2.4, 0.0, 0.5, 0, 240, 0}
	};

	GLuint pieceList = drawPeg();
	unsigned char num_hit_pegs = MAX_HITS - player[pnum].num_hits;
	unsigned char num_miss_pegs = (MAX_MISSES - (MAX_HITS) - player[pnum].num_misses)/2;

	//red pegs
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
	glPushMatrix();
	size_t i;
	for (i = 0; i < num_hit_pegs; i++)
	{
		glTranslatef(red[i][0], red[i][1], red[i][2]);
		glRotatef(red[i][3], 1, 0, 0);
		glRotatef(red[i][4], 0, 1, 0);
		glRotatef(red[i][5], 0, 0, 1);
		glCallList(pieceList);
	}
	glPopMatrix();
	
	//white pegs
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	glPushMatrix();
	glPushMatrix();
	glTranslatef(0.0, 0.0, SLOT_HEIGHT - GRID_BOTTOM_Z);
	size_t j;
	for (i = 0, j = 0; i < MAX_HITS && j < num_miss_pegs; i++, j++)
	{
		glTranslatef(red[i][0], red[i][1], red[i][2]);
		glRotatef(red[i][3], 1, 0, 0);
		glRotatef(red[i][4], 0, 1, 0);
		glRotatef(red[i][5], 0, 0, 1);
		glCallList(pieceList);
	}
	glPopMatrix();
	glTranslatef(0.0, 0.0, SLOT_HEIGHT);
	for (i = 0; i < MAX_HITS && j < num_miss_pegs; i++, j++)
	{
		glTranslatef(red[i][0], red[i][1], red[i][2]);
		glRotatef(red[i][3], 1, 0, 0);
		glRotatef(red[i][4], 0, 1, 0);
		glRotatef(red[i][5], 0, 0, 1);
		glCallList(pieceList);
	}
	glPopMatrix();
}  

static GLuint 
drawCarrier()
{
	GLfloat pegStart = BLOCK_REAL_DIM/2.0;
	static const GLfloat G = 0.1;

	START_LIST
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_ship);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_ship);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_ship);
		//Pegs
		glPushMatrix();
			glTranslatef(0, -PEG_LEN_2, pegStart);
			glRotatef(-90, 1, 0, 0);
			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
			glTranslatef(0.0, -(BLOCK_REAL_DIM*2), 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
		glPopMatrix();
		glBegin(GL_QUADS);
			//Back
			glNormal3f(0, 0, -1);
			glVertex3f(SHIP_HULL_DIAM*2.0, SHIP_HULL_HEIGHT, G);
			glVertex3f(SHIP_HULL_DIAM*2.0, 0, G);
			glVertex3f(-SHIP_HULL_DIAM*2.0, 0, G);
			glVertex3f(-SHIP_HULL_DIAM*2.0,SHIP_HULL_HEIGHT, G);
			//Left
			glNormal3f(-1, 0, 0);
			glVertex3f(-SHIP_HULL_DIAM*2.0,SHIP_HULL_HEIGHT, G);
			glVertex3f(-SHIP_HULL_DIAM*2.0, 0, G);
			glVertex3f(-SHIP_HULL_DIAM*2.0, 0, BLOCK_REAL_DIM*5.0-G);
			glVertex3f(-SHIP_HULL_DIAM*2.0,SHIP_HULL_HEIGHT,BLOCK_REAL_DIM*5.0-G);
			//Front
			glNormal3f(0, 0, 1);
			glVertex3f(-SHIP_HULL_DIAM*2.0,SHIP_HULL_HEIGHT,BLOCK_REAL_DIM*5.0-G);
			glVertex3f(-SHIP_HULL_DIAM*2.0, 0, BLOCK_REAL_DIM*5.0-G);
			glVertex3f(SHIP_HULL_DIAM*2.0, 0, BLOCK_REAL_DIM*5.0-G);
			glVertex3f(SHIP_HULL_DIAM*2.0,SHIP_HULL_HEIGHT,BLOCK_REAL_DIM*5.0-G);
			//Right
			glNormal3f(1, 0, 0);
			glVertex3f(SHIP_HULL_DIAM*2.0, 0, G);
			glVertex3f(SHIP_HULL_DIAM*2.0, SHIP_HULL_HEIGHT, G);
			glVertex3f(SHIP_HULL_DIAM*2.0, SHIP_HULL_HEIGHT, BLOCK_REAL_DIM*5.0 - G);
			glVertex3f(SHIP_HULL_DIAM*2.0, 0, BLOCK_REAL_DIM*5.0 - G);
		glEnd();
		//The deck
		glprimitive_box(-BLOCK_DIM/2.0, SHIP_HULL_HEIGHT, 0, 
						BLOCK_DIM, SHIP_DECK_HEIGHT, BLOCK_REAL_DIM*5);
		//Controll Tower
		static const GLfloat G2 = 0.2;
		glprimitive_box(-PEG_DIAM_1, 
						SHIP_HULL_HEIGHT + SHIP_DECK_HEIGHT, 
						BLOCK_REAL_DIM*2,
						G2, G2, BLOCK_REAL_DIM);
		glprimitive_box(-PEG_DIAM_1, 
						SHIP_HULL_HEIGHT + SHIP_DECK_HEIGHT + G2, 
						BLOCK_REAL_DIM*2.5 - G2/2.0,
						G2, G2, G2);
	END_LIST	
}

static GLuint drawBattleship()
{
	GLfloat pegStart = BLOCK_REAL_DIM/2.0;

	START_LIST
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_ship);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_ship);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_ship);
		//Pegs
		glPushMatrix();
			glTranslatef(0, -PEG_LEN_2, pegStart);
			glRotatef(-90, 1, 0, 0);
			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM*2.0);
			glRotatef(-90, 1.0, 0.0, 0.0);
			//The hull
			glPushMatrix();
				glScalef(1.0, BLOCK_REAL_DIM*6.0, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.1, SHIP_HULL_HEIGHT);
			glPopMatrix();	
			//The deck
			glPushMatrix();
				glTranslatef(0.0, 0.0, SHIP_HULL_HEIGHT);
				glScalef(1.0, BLOCK_REAL_DIM*6.0, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.2, SHIP_DECK_HEIGHT);
				glTranslatef(0.0, 0.0, SHIP_DECK_HEIGHT);
				glprimitive_disk(SHIP_HULL_DIAM*2.2);
			glPopMatrix();	
		glPopMatrix();	
		//Controll Tower
		static const GLfloat G2 = 0.2;
		glprimitive_box(-G2/2.0, 
						SHIP_HULL_HEIGHT + SHIP_DECK_HEIGHT, 
						BLOCK_REAL_DIM*2 - BLOCK_REAL_DIM/2.0,
						G2, G2, BLOCK_REAL_DIM);
	END_LIST
}

static GLuint drawDestroyer()
{
	GLfloat pegStart = BLOCK_REAL_DIM/2.0;

	START_LIST
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_ship);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_ship);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_ship);
		//Pegs
		glPushMatrix();
			glTranslatef(0, -PEG_LEN_2, pegStart);
			glRotatef(-90, 1, 0, 0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
			glTranslatef(0.0, -BLOCK_REAL_DIM*2.0, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM*1.5);
			glRotatef(-90, 1.0, 0.0, 0.0);
			//The hull
			glPushMatrix();
				glScalef(1.0, BLOCK_REAL_DIM*4.5, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.0, SHIP_HULL_HEIGHT);
			glPopMatrix();	
			//The deck
			glPushMatrix();
				glTranslatef(0.0, 0.0, SHIP_HULL_HEIGHT);
				glScalef(1.0, BLOCK_REAL_DIM*4.5, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.1, SHIP_DECK_HEIGHT);
				glTranslatef(0.0, 0.0, SHIP_DECK_HEIGHT);
				glprimitive_disk(SHIP_HULL_DIAM*2.1);
			glPopMatrix();	
		glPopMatrix();	
		//Controll Tower
		static const GLfloat G2 = 0.2;
		glprimitive_box(-G2/2.0, 
						SHIP_HULL_HEIGHT + SHIP_DECK_HEIGHT, 
						BLOCK_REAL_DIM,
						G2, G2, G2);
		glprimitive_box(-G2/2.0, 
						SHIP_HULL_HEIGHT + SHIP_DECK_HEIGHT, 
						BLOCK_REAL_DIM + PEG_DIAM_1*3,
						G2, G2, G2);
	END_LIST
}

static GLuint drawSub()
{
	GLfloat pegStart = BLOCK_REAL_DIM/2.0;

	START_LIST
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_ship);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_ship);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_ship);
		//Pegs
		glPushMatrix();
			glTranslatef(0, -PEG_LEN_2, pegStart);
			glRotatef(-90, 1, 0, 0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
			glTranslatef(0.0, -BLOCK_REAL_DIM*2.0, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
		glPopMatrix();
		glTranslatef(0.0, (SHIP_HULL_HEIGHT-SHIP_DECK_HEIGHT)/2.0, 0.0);
		//Hull
		glPushMatrix();
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM*1.5);
			glRotatef(-90, 1.0, 0.0, 0.0);
			glPushMatrix();
				glScalef(1.0, BLOCK_REAL_DIM*4.5, 
						(SHIP_HULL_HEIGHT+SHIP_DECK_HEIGHT)*2.5);
				glprimitive_sphere(SHIP_HULL_DIAM*2.0);
			glPopMatrix();	
		glPopMatrix();	
		//Controll Tower
		static const GLfloat G2 = 0.2;
		glprimitive_box(-G2/2.0, 
						SHIP_HULL_HEIGHT, 
						BLOCK_REAL_DIM,
						G2, G2, G2*2);
	END_LIST
}

static GLuint drawPT()
{
	GLfloat pegStart = BLOCK_REAL_DIM/2.0;

	START_LIST
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_ship);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_ship);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_ship);
		//Pegs
		glPushMatrix();
			glTranslatef(0, -PEG_LEN_2, pegStart);
			glRotatef(-90, 1, 0, 0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
			glprimitive_cylinder(PEG_DIAM_2, PEG_LEN_2);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM);
			glRotatef(-90, 1.0, 0.0, 0.0);
			//The hull
			glPushMatrix();
				glScalef(1.0, BLOCK_REAL_DIM*3.0, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.0, SHIP_HULL_HEIGHT);
			glPopMatrix();	
			//The deck
			glPushMatrix();
				glTranslatef(0.0, 0.0, SHIP_HULL_HEIGHT);
				glScalef(1.0, BLOCK_REAL_DIM*3.0, 1.0);
				glprimitive_cylinder(SHIP_HULL_DIAM*2.1, SHIP_DECK_HEIGHT);
				glTranslatef(0.0, 0.0, SHIP_DECK_HEIGHT);
				glprimitive_disk(SHIP_HULL_DIAM*2.1);
			glPopMatrix();	
		glPopMatrix();	
		static const GLfloat G2 = 0.2;
		glprimitive_box(-G2/2.0, 
						SHIP_HULL_HEIGHT+SHIP_DECK_HEIGHT, 
						BLOCK_REAL_DIM-G2,
						G2, G2, 2*G2);
	END_LIST
}

static GLuint
drawShip(enum battleship_shiptype snum)
{
	GLuint r = 0;
	switch (snum)
	{
		case CARRIER: r = drawCarrier(); break;
		case BATTLESHIP: r = drawBattleship(); break;
		case DESTROYER: r = drawDestroyer(); break;
		case SUB: r = drawSub(); break;
		case PT: r = drawPT(); break;
		default:
			//DPRINT("Invalid ship to drawShip %i\n", snum);
		break;
	}
	return r;
}

static void
drawShips(unsigned char pnum)
{
	static const GLfloat SHIP_LOC[NUM_SHIPS][6] =
	{
		{-BLOCK_REAL_DIM/2.0, 0, 0, 0, 0, 0},
		{-BLOCK_REAL_DIM/2.0-BLOCK_REAL_DIM, 0, BLOCK_REAL_DIM*3, 0, 0, 0},
		{-BLOCK_REAL_DIM/2.0-BLOCK_REAL_DIM*2, 0, 0, 0, 0, 0},
		{-BLOCK_REAL_DIM/2.0, 0, BLOCK_REAL_DIM*(MAX_SHIP_LEN+1), 0, 0, 0},
		{-BLOCK_REAL_DIM/2.0-BLOCK_REAL_DIM*2, 0, BLOCK_REAL_DIM*MAX_SHIP_LEN+1, 0, 0, 0}
	};
		
	size_t i;
	for (i = 0; i < NUM_SHIPS; i++)
	{
		glPushMatrix();

		//move to orign of grid
		glTranslatef(-(GRID_DIM/2.0)*BLOCK_REAL_DIM, 
				  GRID_BOTTOM_Y + BLOCK_DEPTH/2.0, 
				  GRID_BOTTOM_Z + BLOCK_REAL_DIM/2.0);

		//draw ships that are on the grid
		if (player[pnum].ship[i].state != SHIP_NOT_PLACED)
		{
			if (player[pnum].ship[i].state == SHIP_PLACING)
			{
				glTranslatef(0.0, 1.0, 0.0);
			}
			
			if (player[pnum].ship[i].down)
			{
				//translate to right grid
				glTranslatef(
					player[pnum].ship[i].x*BLOCK_REAL_DIM + BLOCK_REAL_DIM/2.0,
					0.0,
					player[pnum].ship[i].y*BLOCK_REAL_DIM - BLOCK_REAL_DIM/2.0);
			}
			else
			{
				glTranslatef(
					player[pnum].ship[i].x*BLOCK_REAL_DIM,
					0.0,
					player[pnum].ship[i].y*BLOCK_REAL_DIM);
				//rotate ship
				glRotatef(90, 0.0, 1.0, 0.0);
			}
		}
		//draw ships that are in the holder
		else
		{
			//move to other players holder
			glTranslatef(SHIP_LOC[i][0], SHIP_LOC[i][1], SHIP_LOC[i][2]);
			glRotatef(SHIP_LOC[i][3], 1, 0, 0);
			glRotatef(SHIP_LOC[i][4], 0, 1, 0);
			glRotatef(SHIP_LOC[i][5], 0, 0, 1);
		}
	
		//don't draw the ship if fog is on
		if (!player[pnum].fog || 
			(game_state == GAME_PLACE_SHIPS && pnum == curr_player))
		{
			glCallList(drawShip(i));
		}

		//draw hit pegs
		  //translate to center of square
		if (player[pnum].ship[i].down)
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM/2.0);
		else
			glTranslatef(0.0, 0.0, BLOCK_REAL_DIM/2.0);
		glRotatef(-90, 1, 0, 0);

		size_t j;
		int hscale = (player[pnum].ship[i].down) ? 0 : 1;
		int vscale = (player[pnum].ship[i].down) ? 1 : 0;

		//get peg ready
		GLuint peg = drawPeg();
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);

		for (j = 0; j < player[pnum].ship[i].length; j++)
		{
			if (player[pnum].grid
					[player[pnum].ship[i].x + j*hscale]
					[player[pnum].ship[i].y + j*vscale]
					== GRID_HIT
			   )
			{
				glCallList(peg);
			}
			glTranslatef(0.0, -BLOCK_REAL_DIM, 0.0);
		}

		glPopMatrix();
	}
}

static void drawFog()
{
	if (do_fog == FOG_OFF) return;
	if (!do_textures) glEnable(GL_TEXTURE_2D);

	double d = GRID_DIM*BLOCK_REAL_DIM;
	double s = 1.0/(d + 2.0*FOG_HEIGHT);
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);

	//Generate (if needed) and load the fog texture
	loadFog();

	glBegin(GL_QUADS);
		//Top
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(FOG_HEIGHT*s, FOG_HEIGHT*s);
		glVertex3f(d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f(FOG_HEIGHT*s, (FOG_HEIGHT + d)*s);
		glVertex3f(d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT+d)*s, (FOG_HEIGHT + d)*s);
		glVertex3f(-d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT+d)*s, FOG_HEIGHT*s);
		glVertex3f(-d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		//Left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(FOG_HEIGHT*s, FOG_HEIGHT*s);
		glVertex3f(d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f(0, FOG_HEIGHT*s);
		glVertex3f(d/2.0, 0.0, BLOCK_DEPTH/2.0);
		glTexCoord2f(0, (FOG_HEIGHT + d)*s);
		glVertex3f(d/2.0, d, BLOCK_DEPTH/2.0);
		glTexCoord2f(FOG_HEIGHT*s, (FOG_HEIGHT+d)*s);
		glVertex3f(d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		//Right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, FOG_HEIGHT*s);
		glVertex3f(-d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, (FOG_HEIGHT + d)*s);
		glVertex3f(-d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f(1, (FOG_HEIGHT+d)*s);
		glVertex3f(-d/2.0, d, BLOCK_DEPTH/2.0);
		glTexCoord2f(1, FOG_HEIGHT*s);
		glVertex3f(-d/2.0, 0.0, BLOCK_DEPTH/2.0);
		//Back
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, 0);
		glVertex3f(-d/2.0, 0.0, BLOCK_DEPTH/2.0);
		glTexCoord2f(FOG_HEIGHT*s, 0);
		glVertex3f(d/2.0, 0.0, BLOCK_DEPTH/2.0);
		glTexCoord2f(FOG_HEIGHT*s, FOG_HEIGHT*s);
		glVertex3f(d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, FOG_HEIGHT*s);
		glVertex3f(-d/2.0, 0.0, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		//Front
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(FOG_HEIGHT*s, (FOG_HEIGHT + d)*s);
		glVertex3f(d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
		glTexCoord2f(FOG_HEIGHT*s, 1);
		glVertex3f(d/2.0, d, BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, 1);
		glVertex3f(-d/2.0, d, BLOCK_DEPTH/2.0);
		glTexCoord2f((FOG_HEIGHT + d)*s, (FOG_HEIGHT+d)*s);
		glVertex3f(-d/2.0, d, FOG_HEIGHT + BLOCK_DEPTH/2.0);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
	
	if (!do_textures) glDisable(GL_TEXTURE_2D);
}

static void
drawMenu()
{
	GLfloat size = 1.5;
	glPushMatrix();

	//Center title horizontally (and put it at top) 
	GLdouble w = glfont_char_width(size)*strlen(curr_menu->name)/2.0;
	glPushMatrix();
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
    	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
		
		glTranslatef(-w, FONT_Y0, 0.0);
		glfont_draw_string(curr_menu->name, size);
		
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
    	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	glPopMatrix();	

	//find max lengthed menu item
	size_t max_len = 0;
	size_t len = 0;
	size_t i;
	for (i = 0; i < curr_menu->size; i++)
	{
		if (!curr_menu->item[i]) continue;
		len = strlen(curr_menu->item[i]->name);
		if (curr_menu->item[i]->svalue)
			len += strlen(curr_menu->item[i]->svalue) + 2;
		max_len = (max_len < len) ? len : max_len;
	}
	
	//Options are smaller than title
	size = 1.0;
	//Center items
	w = glfont_char_width(size)*max_len/2.0;
	GLdouble h = glfont_char_height(size);
	glTranslatef(-w, h*curr_menu->size/2.0 + h*(curr_menu->size + 1)/2.0, 0.0);
	
	w = glfont_char_width(size);
	h *= 2.0; //double space
	//double space
	for (i = 0; i < curr_menu->size; i++)
	{
		glTranslatef(0.0, -h, 0.0);
		
		//separator
		if (!curr_menu->item[i]) 
		{ 
			glPushMatrix();
				size_t j;
				for (j = 0; j < max_len - 1; j++)
				{
					glfont_draw_string("-", size); 
					glTranslatef(w, 0.0, 0.0);
				}
			glPopMatrix();
			continue; 
		}
		
		//draw selection
		if (curr_menu_sel == i)
		{
			glPushMatrix();
				glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
    			glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
				glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, h/4.0, w/4.0);
				glCallList(drawPeg());
				glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
    			glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
				glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
			glPopMatrix();
		}
		
		//draw name
		glfont_draw_string(curr_menu->item[i]->name, size);
		  //draw value
		if (curr_menu->item[i]->svalue)
		{
			glPushMatrix();
				glTranslatef(w * strlen(curr_menu->item[i]->name), 0.0, 0.0);
				glfont_draw_string(":", size);
				glTranslatef(w, 0.0, 0.0);
				glfont_draw_string(curr_menu->item[i]->svalue, size);
			glPopMatrix();	
		}
		  //draw plus for submenus
		else if (curr_menu->item[i]->size > 0)
		{
			glPushMatrix();
				glTranslatef(w * strlen(curr_menu->item[i]->name), 0.0, 0.0);
				glfont_draw_string("+", size);
			glPopMatrix();	
		}
	}
	glPopMatrix();

	//Help text
	glPushMatrix();
		size = 0.8;
		w = glfont_char_width(size);
		char *text = NULL;
		if (curr_menu->item[curr_menu_sel]->svalue)
			text = "(Left/Right to change)";
		else
			text = "(Enter to select)";
		//glTranslatef(0.0, 0.0, 0.0);
		glTranslatef(-w*strlen(text)/2.0, FONT_Y1, 0.0);
		glfont_draw_string(text, size);
	glPopMatrix();
}

static void drawNameSelector()
{
	GLdouble size = 1.5;

	//Center title horizontally (and put it at top) 
	glPushMatrix();
		char *text = "Select Name";
		GLdouble w = glfont_char_width(size)*strlen(text)/2.0;
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
			
		//title
		glTranslatef(-w, FONT_Y0, 0.0);
		glfont_draw_string(text, size);
			
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
		glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);

		//draw name
		size = 1.2;
		GLdouble w2 = glfont_char_width(size)*MAX_NAME_LEN/2.0;
		GLdouble h = -2.5 * glfont_char_height(size);
		glTranslatef(w - w2, h, 0.0);
		glfont_draw_string(name_selector.name, size);

		//Draw bar under current letter
		GLdouble g = 0.2;
		size_t len = strlen(name_selector.name);
		if (len < MAX_NAME_LEN)
			glprimitive_box(len * glfont_char_width(size), 0.0, 0.0,
							glfont_char_width(size), g, g);
		
		//draw box around name
		h = glfont_char_height(size);
		GLdouble diff = h - size;
		 //left
		glprimitive_box(-size/2.0 - g, -size/2.0 - g, 0.0,
						g, h + size - diff + g*2.0, g);
		 //right
		glprimitive_box(2.0*w2 + size/2.0, -size/2.0 - g, 0.0,
							g, h + size - diff + g*2.0, g);
		 //top
		glprimitive_box(-size/2.0, h + size/2.0 - diff, 0.0,
						 2.0*w2 + size, g, g);
		 //bottom
		glprimitive_box(-size/2.0, -size/2.0 - g, 0.0,
						 2.0*w2 + size, 0.2, g);
	glPopMatrix();	
	
	glPushMatrix();
		size = 2.0;
		glTranslatef(-GLFONT_TEST_WIDTH*glfont_char_width(size)/2.0,
					  glfont_char_height(size),
					  0.0);
		glfont_test(size, name_selector.x, name_selector.y);
	glPopMatrix();
}

static void drawStatus()
{
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	
	glPushMatrix();
	if (game_state == GAME_PLACE_SHIPS) 
	{
		GLfloat size = 0.8;
		GLfloat w = glfont_char_width(size);
		char *text = NULL; 

		//Draw text
		text = "Place Your Ships";
		size_t len = strlen(text);
		glTranslatef(-w*len/2.0, glfont_char_height(size)*1.5, 5.0);
		glfont_draw_string(text, size);

		//Draw player name
		size_t len2 = strlen(player[curr_player].name);
		glTranslatef(w*(len/2.0 - len2/2.0), glfont_char_height(size)*2.0, 0.0);
		glfont_draw_string(player[curr_player].name,  size);
	}
	else if (game_state == GAME_PLAYING || game_state == GAME_AI_PLAYING)
	{
		GLfloat size = 0.8;
		GLfloat w = glfont_char_width(size);

		//Draw message
		glTranslatef(FONT_X0, FONT_Y0 + 1, 0.0);
		glfont_draw_string("Ready to fire!", size);
		//Draw grid coordinate
		glTranslatef(0.0, -glfont_char_height(size)*2.0, 0.0);
		char c[6] = {'(', '\0', '-', '\0', ')', '\0'};
		c[1] = player[curr_player].sel_x + 65;
		c[3] = player[curr_player].sel_y + 48;
		glfont_draw_string(c, size);
		
		//Draw player name
		size_t len = strlen(player[curr_player].name);
		glTranslatef(-FONT_X0 + FONT_X1 - len* w, 
					 glfont_char_height(size)*2.0, 0.0);
		glfont_draw_string(player[curr_player].name, size);
		glTranslatef(len*w, -glfont_char_height(size)*2.0, 0.0);
	
		//Draw player type
		int ait = (int) player[curr_player].ai;
		len = strcspn(battleship_aitype_s[ait], " "); 
		glTranslatef(-w*len, 0.0, 0.0);
		glfont_draw_string((char *) battleship_aitype_s[ait], size);
	}
	else if (game_state == GAME_MESSAGE || game_state == GAME_OVER)
	{
		static const GLfloat diff_water[] = {0.0, 0.2, 0.9, 0.8};
		static const GLfloat spec_water[] = {0.0, 0.0, 0.0, 0.8};
		static const GLfloat shinny_water = 1;
		
		GLfloat size = 0.4*game_message.size;
		GLfloat w = glfont_char_width(size);
	
		if (game_message.type == GRID_MISS)
		{
			glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_water);
    		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_water);
			glMaterialf(GL_FRONT, GL_SHININESS, shinny_water);
		}
		else
		{
			glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_r);
    		glMaterialfv(GL_FRONT, GL_SPECULAR, spec_r);
			glMaterialf(GL_FRONT, GL_SHININESS, shinny_r);
		}
		
		glTranslatef(0.0, 0.0, GRID_DIM*BLOCK_REAL_DIM + SLOT_WIDTH);
		char *text = NULL;
		if (game_message.ship)
		{
			text = "Sunk!";
			size_t len = strlen(game_message.ship);
			glTranslatef(-w*len/2.0, 
						 glfont_char_height(size)*1.5, 
						 0.0);
			glfont_draw_string(game_message.ship, size);
			glTranslatef(w*len/2.0 - w*strlen(text)/2.0,
						 -glfont_char_height(size)*2.0,
						 0.0);
			glfont_draw_string(text, size);
		}
		else if (game_message.type == GRID_EMPTY)
		{
			//Game over
			text = "Game Over";
			size_t len = strlen(text);
			glTranslatef(-w*len/2.0, 
						 glfont_char_height(size)*2.5, 
						 0.0);
			glfont_draw_string(text, size);
			//Players name
			glTranslatef(w*len/2.0 - w*strlen(player[curr_player].name)/2.0,
						 -glfont_char_height(size)*2.0,
						 0.0);
			glfont_draw_string(player[curr_player].name, size);
			//Wins
			text = "Wins!";
			glTranslatef(w*strlen(player[curr_player].name)/2.0
						 -w*strlen(text)/2.0,
						 -glfont_char_height(size)*2.0,
						 0.0);
			glfont_draw_string(text, size);
		}
		else
		{
			if (game_message.type == GRID_MISS)
				text = "Miss";
			else
				text = "Hit";
		
			glTranslatef(-w*strlen(text)/2.0, 0.0, 0.0);
			glfont_draw_string(text, size);
		}
	}
	glPopMatrix();
}

static void drawClock(bool test)
{
	glPushMatrix();

	if (test)
	{
		glprimitive_clock(8.0, 15);
	}
	else
	{
		glTranslatef(
				-GRID_DIM/2.0 * BLOCK_REAL_DIM - SLOT_WIDTH - 2, 
				0.0, 
				1 + GRID_BOTTOM_Z
				);
			glRotatef(40, 0.0, 1.0, 0.0);
			glTranslatef(-1.0, 0.2, 0.0);
		glprimitive_clock(2.0, 15);
	}

	glPopMatrix();
}

static void drawMug(bool test)
{
	glPushMatrix();

	double size = 0;
	if (test)
	{
		size = 8.0;
		glprimitive_mug(size, 15);
	}
	else
	{
		size = 2.0;
		glTranslatef(
				GRID_DIM/2.0 * BLOCK_REAL_DIM + SLOT_WIDTH + 2, 
				0.0, 
				-2 - GRID_BOTTOM_Z
				);
		glRotatef(30, 0.0, 1.0, 0.0);
		glprimitive_mug(size, 15);
	}
	
	//draw coffie
	static const GLfloat diff[] = {0.6, 0.3, 0.0, 1.0};
	static const GLfloat spec[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat shinny = 1.0;
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny);
	glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_COFFIE]);
	
    glRotatef(-90, 1.0, 0.0, 0.0);
    glTranslatef(0.0, 0.0, size);
	glprimitive_disk(size - size/4.0);

	glBindTexture(GL_TEXTURE_2D, 0);

	glPopMatrix();
}

static void drawPicture(unsigned char pnum)
{
	static const GLfloat diff[] = {0.6, 0.4, 0.0, 1.0};
	static const GLfloat spec[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat shinny = 1.0;
	
	glPushMatrix();

	if (pnum == 0)
		glTranslatef(20, 6, ROOM_DIM/2.0 - 0.1);
	else 
		glTranslatef(-20, 8, ROOM_DIM/2.0 - 0.1);
	
	//frame
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny);
	glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_TABLE]);
		//left
		glprimitive_box(-PICTURE_DIM/2.0 - PICTURE_FRAME_R,
						-PICTURE_DIM/2.0 - PICTURE_FRAME_R,
						-PICTURE_FRAME_R, 
						PICTURE_FRAME_R, 
						PICTURE_DIM + PICTURE_FRAME_R*2.0, 
						PICTURE_FRAME_R);
		//right
		glprimitive_box(PICTURE_DIM/2.0, 
						-PICTURE_DIM/2.0 - PICTURE_FRAME_R,
						-PICTURE_FRAME_R, 
						PICTURE_FRAME_R, 
						PICTURE_DIM + PICTURE_FRAME_R * 2.0, 
						PICTURE_FRAME_R);
		//top
		glprimitive_box(-PICTURE_DIM/2.0, 
						PICTURE_DIM/2.0, 
						-PICTURE_FRAME_R, 
						PICTURE_DIM,
						PICTURE_FRAME_R, PICTURE_FRAME_R);
		//top
		glprimitive_box(-PICTURE_DIM/2.0, 
						-PICTURE_DIM/2.0 - PICTURE_FRAME_R, 
						-PICTURE_FRAME_R, 
						PICTURE_DIM,
						PICTURE_FRAME_R, PICTURE_FRAME_R);
	
	//picture
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_w);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_w);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_w);
	if (pnum == 0)
		glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_PIC1]);
	else
		glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_PIC2]);
	glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0, 0); 
		glVertex3f(-PICTURE_DIM/2.0, PICTURE_DIM/2.0, -PICTURE_FRAME_R/2.0);
		glTexCoord2f(0, 1); 
		glVertex3f(-PICTURE_DIM/2.0, -PICTURE_DIM/2.0, -PICTURE_FRAME_R/2.0);
		glTexCoord2f(1, 1); 
		glVertex3f(PICTURE_DIM/2.0, -PICTURE_DIM/2.0, -PICTURE_FRAME_R/2.0);
		glTexCoord2f(1, 0); 
		glVertex3f(PICTURE_DIM/2.0, PICTURE_DIM/2.0, -PICTURE_FRAME_R/2.0);
	glEnd();
	
	glBindTexture(GL_TEXTURE_2D, 0);

	glPopMatrix();
}

static void drawTable()
{
	GLfloat ypos = -TABLE_HEIGHT + TABLE_TOP; 
	//top
	glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_TABLE]);
	glprimitive_box(-TABLE_WIDTH/2.0, 
					ypos,
					-TABLE_DEPTH/2.0,
					 TABLE_WIDTH, TABLE_HEIGHT, TABLE_DEPTH);
	glBindTexture(GL_TEXTURE_2D, 0);
	
	//legs
	static const GLfloat diff[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat spec[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat shinny = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny);
	GLdouble r = 0.5;
	GLdouble b = 3.0;
	size_t i;
	for (i = 0; i < 2; i++)
	{
		glPushMatrix();
			if (i == 1) glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-TABLE_WIDTH/2.0 + b,
						 ypos - r,
						 -TABLE_DEPTH/2.0 + b);
			glprimitive_cylinder(r, TABLE_DEPTH - 2.0*b);
			glTranslatef(0.0, 0.0, 3.0*r);
			glRotatef(90, 1.0, 0.0, 0.0);
			glprimitive_cylinder(r, ROOM_HEIGHT/2.0 + ypos - r);
			glTranslatef(0.0, TABLE_DEPTH - 2.0*b - 6.0*r, 0.0);
			glprimitive_cylinder(r, ROOM_HEIGHT/2.0 + ypos -r);
		glPopMatrix();
	}
}

static void drawCeiling()
{
	GLfloat s = ROOM_DIM/5.0;
	glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_CEILING]);
	glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0, 0); 
		glVertex3f(-ROOM_DIM/2.0, ROOM_HEIGHT/2.0, -ROOM_DIM/2.0);
		glTexCoord2f(s, 0); 
		glVertex3f(ROOM_DIM/2.0, ROOM_HEIGHT/2.0, -ROOM_DIM/2.0);
		glTexCoord2f(s, s); 
		glVertex3f(ROOM_DIM/2.0, ROOM_HEIGHT/2.0, ROOM_DIM/2.0);
		glTexCoord2f(0, s); 
		glVertex3f(-ROOM_DIM/2.0, ROOM_HEIGHT/2.0, ROOM_DIM/2.0);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);

	//draw light
	GLfloat g = 3.0;
	glprimitive_box(LIGHT_X - g/2.0, LIGHT_Y - 0.5, LIGHT_Z - g/2.0, g, 0.5, g);
}

static void drawFloor()
{
	GLfloat s = ROOM_DIM/5.0;
	glBindTexture(GL_TEXTURE_2D, texture_id[TEXTURE_FLOOR]);
	glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0, s); 
		glVertex3f(-ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, ROOM_DIM/2.0);
		glTexCoord2f(s, s); 
		glVertex3f(ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, ROOM_DIM/2.0);
		glTexCoord2f(s, 0); 
		glVertex3f(ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, -ROOM_DIM/2.0);
		glTexCoord2f(0, 0); 
		glVertex3f(-ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, -ROOM_DIM/2.0);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
}

static void drawWall()
{
	if (!do_textures) glEnable(GL_TEXTURE_2D);

	//Generate (if needed) and load the lsystem texture
	if (game_lsys.enabled)
		loadLsystem();

	glPushMatrix();
		glTranslatef(0.0, 0.0, ROOM_DIM/2.0);
		glBegin(GL_QUADS);
			glNormal3f(0.0, 0.0, -1.0);
			glTexCoord2f(1.0, 0.0);
			glVertex3f(-ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, 0.0);
			glTexCoord2f(0.0, 0.0);
			glVertex3f(ROOM_DIM/2.0, -ROOM_HEIGHT/2.0, 0.0);
			glTexCoord2f(0.0, 1.0);
			glVertex3f(ROOM_DIM/2.0, ROOM_HEIGHT/2.0, 0.0);
			glTexCoord2f(1.0, 1.0);
			glVertex3f(-ROOM_DIM/2.0, ROOM_HEIGHT/2.0, 0.0);
		glEnd();
	glPopMatrix();	
	glBindTexture(GL_TEXTURE_2D, 0);
	
	if (!do_textures) glDisable(GL_TEXTURE_2D);
}

/*
GLdouble points[] = 
{
	//xy
//	0.0, -5.0, 0.0, 
//	5.0, 0.0, 0.0, 
//	0.0, 5.0, 0.0,
//	-5.0, 0.0, 0.0,
//	0.0, -5.0, 0.0
	
	//yz
	5,
	0.0, -5.0, 0.0, 
	0.0, 0.0, -5.0, 
	0.0, 5.0, 0.0,
	0.0, 0.0, 5.0,
	0.0, -5.0, 0.0
	
	//xz
//	-5.0, 0.0, 0.0, 
//	0.0, 0.0, 5.0, 
//	5.0, 0.0, 0.0,
//	0.0, 0.0, -5.0,
//	-5.0, 0.0, 0.0
	
//	0.0, 0.0, -5.0, 
//	-5.0, 5.0, 0.0, 
//	0.0, 0.0, 5.0, 
//	5.0, -5.0, 0.0, 
//	0.0, 0.0, -5.0, 
};
*/

void battleship_view_ready_rocket()
{
	if (game_rocket.type == ROCKET_OFF) return;

	GLdouble *path = rocket_path[game_rocket.type];

	path[(PATH_SIZE*3 + 1) - 1] = 
	-(GRID_BOTTOM_Z + BLOCK_REAL_DIM * (double) player[curr_player].sel_y)
	- BLOCK_REAL_DIM*0.5;
	path[(PATH_SIZE*3 + 1) - 3] = 
	(GRID_DIM*0.5 - player[curr_player].sel_x)*BLOCK_REAL_DIM
	- BLOCK_REAL_DIM*0.5;

	battleship_rocket_set_path(path);	
}

static void drawRocket()
{
	if (game_rocket.type == ROCKET_OFF) return;

	battleship_rocket_draw(game_rocket.show_path || do_test == TEST_ROCKET);
}

static GLuint loadTextureFile(char *filename, int w, int h)
{
    GLuint texture;

    FILE *thef = fopen(filename, "rb");
    if (thef == NULL)
	{
		//DPRINT("Error loading texture  %s\n", filename);
		return 0;
	}

    void *data = malloc(w * h * 3);
    fread(data, w * h * 3, 1, thef);
    fclose(thef);

    glGenTextures(1, &texture);
    glBindTexture(GL_TEXTURE_2D, texture);

    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
					GL_LINEAR_MIPMAP_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
//    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
//					GL_NEAREST);
//    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
//					GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, w, h,
                      GL_RGB, GL_UNSIGNED_BYTE, data);
//  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 
//			 0, GL_RGBA, GL_UNSIGNED_BYTE, data);

    free(data);
    return texture;
}

static GLuint loadTextureData(void *data, int w, int h, bool alpha)
{
	GLuint texture;
	
	glGenTextures(1, &texture);
    glBindTexture(GL_TEXTURE_2D, texture);

    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
					GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
					GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	if (alpha)
	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 
					 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
	else
	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 
					 0, GL_RGB, GL_UNSIGNED_BYTE, data);

	return texture;
}

static void loadLsystem()
{
	static struct battleship_lsystem last; 
	
	if (!memcmp(&last, &game_lsys, sizeof(last)))
	{
		glBindTexture(GL_TEXTURE_2D, lsys_texture);
		return;
	}
	memcpy(&last, &game_lsys, sizeof(last));

	static const int w = 512;
	static const int h = 512;
    void *data = lsystem_test(game_lsys.type, game_lsys.length, w, h);

	//load the texture into memory
	glDeleteTextures(1, &lsys_texture);
	lsys_texture = loadTextureData(data, w, h, false);

	free(data);
}

static void loadFog()
{
	static unsigned int last_type = FOG_OFF;
	if (last_type == do_fog)
	{
		glBindTexture(GL_TEXTURE_2D, fog_texture);
		return;
	}

	if (do_fog == FOG_REGEN)
		do_fog = last_type;
	else
		last_type = do_fog;

	static const int w = 128;
	static const int h = 128;
    char *data = malloc(w * h * 4);
	
	//make fog
		//ready perlin
	struct perlin p;
	p.freq = 0.15;
	p.pers = 0.65;
	p.octaves = 4;
	sperlin(time(0));
	  //deterine density
	double density;
	switch (do_fog)
	{
		case FOG_REGEN:
		case FOG_OFF: density = 0.0; break;
		case FOG_LIGHT: density = 255.0; break;
		case FOG_MEDIUM: density = 200.0; break;
		case FOG_HEAVY: density = 125.0; p.freq = 0.15; break;
	}
	
	int i, j;
	char *d = data;
	for (i = 0; i < w; i++)
	{
		for (j = 0; j < h; j++)
		{
			double val = perlin2D(p, i, j);
			unsigned char n = (unsigned char) ((val + 1.0)/2.0*density);
			d[0] = 200;
			d[1] = 200;
			d[2] = 200;
			d[3] = 255 - n;
		
			/*
			int cover = 90.0;
			double sharp = 0.5;
			int c = n - cover;
			if (c < 0) c = 0;
			int a = (int) (255.0 - pow(sharp, (double) c) * 255.0);
			d[0] = 200;
			d[1] = 200;
			d[2] = 200;
			d[3] = (unsigned char) a;
			*/
			
			d += 4;
		}
	}

	//load the texture into memory
	glDeleteTextures(1, &fog_texture);
	fog_texture = loadTextureData(data, w, h, true);

	free(data);
}
