/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file battleship_view.h 
 *
 *  These files are responsible for the view part of the program.
 */
#ifndef BATTLEHIP_VIEW_H
#define BATTLEHIP_VIEW_H

#ifdef WIN32
#include <windows.h>
#endif /* WIN32 */

#include <GL/gl.h>
#include <GL/glu.h>

#ifdef WIN32
#include "glui.h"
#endif /* WIN32 */

/** Foreces redraw (implementation dependent)
 *
 *  This function should be implement in the main file corrispoding
 *  to a particualr gui frontend.
 *
 *  This fuction causes the window to redraw.
 *
 */
extern void 
battleship_view_refresh();

/** Swaps buffers (implementation dependent)
 *
 *  This function should be implement in the main file corrispoding
 *  to a particualr gui frontend.
 *
 *  Swaps buffers in double buffer mode
 *
 */
extern void
battleship_view_swap_buffers();

/** Initilizes the view
 *
 *  Does not need to be called directly. Will be called by 
 *  battleship_logic_init().
 */
void 
battleship_view_init();

/** Gets the current size of the window 
 *
 *  Used in some places by the game logic.
 *
 *  @param w The width is stored in this variable
 *  @param h The height is stored in this variable
 */
void
battleship_view_getsize(int *w, int *h);

/** Called when the window is resized
 * 
 *  This function should be called from the main file corrisponding to
 *  a particular gui frontend.
 *
 *  Notifies the view of the new size of the windows.
 */
void 
battleship_view_resize(int w, int h);

/** Called when when the window needs to be drawn 
 * 
 *  This function should be called from the main file corrisponding to
 *  a particular gui frontend.
 *
 *  Notifies the window to draw
 */
void 
battleship_view_main();

/** Translates the view
 *
 *  @param axis 0 = x, 1 = y, 2 = z
 *  @param amount Amount to increase the translation by
 *  @param type u = User translation, g = game translation
 *  (captial U and G to increment instead of set)
 */
void battleship_view_set_translate(int axis, float amount, unsigned char type);

/** Rotates the view
 *
 *  @param axis 0 = x, 1 = y, 2 = z
 *  @param amount Amount to increase the rotation by
 *  @param type u = User rotate, g = game rotate 
 *  (captial U and G to increment instead of set)
 */
void battleship_view_set_rotate(int axis, float amount, unsigned char type);

/** Returns the current view translation
 *
 *  @param axis 0 = x, 1 = y, 2 = z
 *  @param type u = User translation, g = game translation
 *  @return The current translation for the given axis
 */
float battleship_view_get_translate(int axis, unsigned char type);

/** Returns the current view rotation
 *
 *  @param axis 0 = x, 1 = y, 2 = z
 *  @param type u = User rotate, g = game rotate
 *  @return The current rotation for the given axis
 */
float battleship_view_get_rotate(int axis, unsigned char type);

/** Gets the rocket ready to be fired base on the current selected
 *  square
 */
void battleship_view_ready_rocket();

#endif /* BATTLEHIP_VIEW_H */
