/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file glfont.c 
 *
 *  These files are responsible for the font.
 */
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <string.h>

#include "glfont.h"
#include "glprimitive.h"

/** The width of pices of the font */
#define WIDTH (size * 0.10)

/** A string containing all the characters of the font, used for 
 *  font test and other things */
static char *font_test_str =
	"ABCDEFGH\nIJKLMNOP\nQRSTUVWX\nYZ012345\n6789.:?!\n()/\\=+- ";

inline void 
glfont_draw_char(char c, GLdouble size)
{
	GLdouble r = WIDTH;
	GLdouble d = 10;
	GLUquadricObj *s = gluNewQuadric();
    gluQuadricDrawStyle(s, GLU_FILL);
		
	c = toupper(c);
	glPushMatrix();
	switch (c)
	{
		case 'A' :
			//Left
			glPushMatrix();
				glRotatef(24, 0.0, 1.0, 0.0);
				glTranslatef(0.0, 0.0, r);
				glprimitive_cylinder(r, size - r);	
				//cap
				gluSphere(s, r, d, d);
			glPopMatrix();
			//Right
			glPushMatrix();
				glTranslatef(size - 2*r, 0.0, 0);
				glRotatef(-24, 0.0, 1.0, 0.0);
				glTranslatef(0.0, 0.0, r);
				glprimitive_cylinder(r, size - r);
				//cap
				gluSphere(s, r, d, d);
			glPopMatrix();
			//Bar
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, 0.0,  2*r);
			glprimitive_cylinder(r, size/2.0 - r);
			//Top cap
			glTranslatef(-size/2.0, 0.0, 2.0*r);
			gluSphere(s, r, d, d);
		break;
		case 'B' :
			//Leg plus caps
			glPushMatrix();
				glTranslatef(0.0, 0.0, r);
				glprimitive_cylinder(r, size - 2*r);	
				gluSphere(s, r, d, d);
				glTranslatef(0.0, 0.0, size - 2*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			//curve of b
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0, 3*r, 0.0);
			glprimitive_half_torus1(r, r);
			glTranslatef(0, size - 6*r, 0.0);
			glprimitive_half_torus1(r, r);
			//Bars of the b
			glRotatef(-90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 2*r, 0);
			glprimitive_cylinder(r, size/2.0);	
			glTranslatef(0.0, -4.0*r, 0.0);
			glprimitive_cylinder(r, size/2.0);	
			glTranslatef(0.0, -4.0*r, 0.0);
			glprimitive_cylinder(r, size/2.0);	
		break;
		case 'C' :
			//C
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			glRotatef(30, 0.0, 0.0, 1.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			glPushMatrix();
				//cop
				glTranslatef(0.0, size/2.0 - r,  0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(-60, 0.0, 0.0, 1.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			//cap
			glTranslatef(0.0, -size/2.0 + r,  0.0);
			gluSphere(s, r, d, d);
		break;
		case 'D' :
			//The leg
			glPushMatrix();
				glTranslatef(0.0, 0.0, r);
				gluSphere(s, r, d, d);
				glprimitive_cylinder(r, size - 2*r);	
				glTranslatef(0.0, 0.0, size - 2*r);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			glPushMatrix();
				//The bars
				glRotatef(90, 0.0, 1.0, 0.0);
				glTranslatef(-r, 0.0, 0.0);
				glprimitive_cylinder(r, size/2.0 - 2*r);	
				glTranslatef(-size + 2*r, 0.0, 0.0);
				glprimitive_cylinder(r, size/2.0 - 2*r);	
			glPopMatrix();
			//The curve
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0 - 2*r, size/2.0, 0.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
		break;
		case 'E' :
			//caps
			glPushMatrix();
				glTranslatef(0.0, 0.0, r);
				//Stick
				glprimitive_cylinder(r, size - 2*r);
				//Cap
				gluSphere(s, r, d, d);
			glPopMatrix();
			glTranslatef(0.0, 0.0, size - r);
			gluSphere(s, r, d, d);
			//Bars
			glTranslatef(size - 2*r, 0.0, 0.0);
			glRotatef(-90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(-size/2.0 + r, 0.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(-size/2.0 + r, 0.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
		break;
		case 'F' :
			//caps
			glPushMatrix();
				glTranslatef(0.0, 0.0, r);
				//Stick
				glprimitive_cylinder(r, size - 2*r);
				//Cap
				gluSphere(s, r, d, d);
			glPopMatrix();
			glTranslatef(0.0, 0.0, size - r);
			gluSphere(s, r, d, d);
			//Bars
			glTranslatef(size - 2*r, 0.0, 0.0);
			glRotatef(-90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(-size/2.0 + r, 0.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
		break;
		case 'G' :
			//caps
			glPushMatrix();
				//The bar
				glRotatef(90, 0.0, 1.0, 0.0);
				glTranslatef(-size/2.0, 0.0, size/2.0 - r);
				glprimitive_cylinder(r, size/2.0 - r);
				//Cap on bar
				gluSphere(s, r, d, d);
			glPopMatrix();
			//top curve
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glRotatef(30, 0.0, 0.0, 1.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			glPushMatrix();
				//cap
				glTranslatef(0.0, size/2.0 - r, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			//Bottom curve
			glRotatef(-120, 0.0, 0.0, 1.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			//cap
			glTranslatef(0.0, -size/2.0 + r, 0.0);
			gluSphere(s, r, d, d);
		break;
		case 'H' :
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			//Right
			glTranslatef(size - 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			glprimitive_cylinder(r, size - 2*r);
			gluSphere(s, r, d, d);
			//Center
			glRotatef(-90, 0.0, 1.0, 0.0);
			glTranslatef(size/2.0 - r, 0.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
		break;
		case 'I' :
			//Middle
			glTranslatef(size/2.0 - r, 0, r);
			glprimitive_cylinder(r, size - 2*r);
			//Bottom
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 0.0, -size/2.0 + r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			//Top
			glTranslatef(-size + 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
		break;
		case 'J' :
			glPushMatrix();
				//center
				glTranslatef(size/2.0 - r, 0, 3*r);
				glprimitive_cylinder(r, size/2.0 + r);
				//curve
				glTranslatef(-2*r, 0.0, 0.0);
				glRotatef(90, 1.0, 0.0, 0.0);
				glRotatef(-90, 0.0, 0.0, 1.0);
				glprimitive_half_torus1(r, r);
				glRotatef(-90, 0.0, 1.0, 0.0);
				//Cap on curve
				glTranslatef(0.0, -2*r, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			//Bottom
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-size + r, 0.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
		break;
		case 'K' :
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			//angly things
			glTranslatef(0.0, 0.0, -size/2.0 + r);
			glRotatef(125, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 3*r);	
			glPushMatrix();
				glTranslatef(0.0, 0.0, size - 3*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(-69, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 3*r);	
			glTranslatef(0.0, 0.0, size - 3*r);
			gluSphere(s, r, d, d);
		break;
		case 'L' :
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);
			gluSphere(s, r, d, d);
			glPushMatrix();
				glTranslatef(0.0, 0.0, size - 2*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 2*r);	
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
		break;
		case 'M' :
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			//caps
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(size - 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			gluSphere(s, r, d, d);
			//Right
			glprimitive_cylinder(r, size - 2*r);	
			glTranslatef(0.0, 0.0, size - 2*r);
			//Middle 
			glPushMatrix();
				glRotatef(225, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0 + r);
			glPopMatrix();	
			glTranslatef(-size + 2*r, 0.0, 0.0);
			glPushMatrix();
				glRotatef(-225, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0 + r);
			glPopMatrix();
			//Middle cap
			glTranslatef(size/2.0 - r, 0.0, -size/2.0 + r);
			gluSphere(s, r, d, d);
		break;
		case 'N' :
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			//caps
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(size - 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			gluSphere(s, r, d, d);
			//Right
			glprimitive_cylinder(r, size - 2*r);	
			//Middle
			glTranslatef(0.0, 0.0, 0.0);
			glRotatef(-45, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size + 1.5*r);	
		break;
		case 'O' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glprimitive_torus(size/2.0 - 2*r, r);
		break;
		case 'P' :
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			//curve
			glTranslatef(size/2.0, 0.0, -2.5*r);
			glRotatef(90, 1.0, 0.0, 0.0);
			glprimitive_half_torus1(1.5*r, r);
			//bars
			glRotatef(-90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 2.5*r, 0.0);
			glprimitive_cylinder(r, size/2.0);
			glTranslatef(0.0, -5*r, 0.0);
			glprimitive_cylinder(r, size/2.0);
		break;
		case 'Q' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glprimitive_torus(size/2.0 - 2*r, r);
			glRotatef(-90, -1.0, 1.0, 0.0);
			//stick
			glTranslatef(0.0, 0.0, 2*r);
			glprimitive_cylinder(r, 3*r);
			//caps
			glTranslatef(0, 0.0, 3*r);
			gluSphere(s, r, d, d);
			glTranslatef(0, 0.0, -3*r);
			gluSphere(s, r, d, d);
		break;
		case 'R' :
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			//curve
			glTranslatef(size/2.0, 0.0, -2.5*r);
			glRotatef(90, 1.0, 0.0, 0.0);
			glprimitive_half_torus1(1.5*r, r);
			//bars
			glRotatef(-90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 2.5*r, 0.0);
			glprimitive_cylinder(r, size/2.0);
			glTranslatef(0.0, -5*r, 0.0);
			glprimitive_cylinder(r, size/2.0);
			//Angled stick	
			glTranslatef(0.0, 0.0, r);
			glRotatef(130, 1.0, 0.0, 0.0);
			glprimitive_cylinder(r, size/2.0 - r);
			glTranslatef(0.0, 0.0, size/2.0 - r);
			gluSphere(s, r, d, d);
		break;
		case 'S' :
			glRotatef(15, 0.0, 1.0, 0.0);
			glTranslatef(-r*1.4, 0.0, r);
			glPushMatrix();
				glRotatef(90, 0.0, 1.0, 0.0);
				glTranslatef(-r, 0.0, 3*r);
				glprimitive_cylinder(r, size - 8*r);
				gluSphere(s, r, d, d);
				glTranslatef(-size/2.0 + r, 0.0, 0.0);
				glprimitive_cylinder(r, size - 8*r);
				glTranslatef(-size/2.0 + r, 0.0, 0.0);
				glprimitive_cylinder(r, size - 8*r);
				glTranslatef(0.0, 0.0, size - 8*r);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			glRotatef(-90, 1.0, 0.0, 0.0);
			glPushMatrix();
				glTranslatef(size - 5*r, -3*r, 0.0);
				glprimitive_half_torus1(r, r);
			glPopMatrix();	
			glPushMatrix();
				glRotatef(180, 0.0, 0.0, 1.0);
				glTranslatef(-3*r, size - 3*r, 0.0);
				glprimitive_half_torus1(r, r);
			glPopMatrix();	
		break;
		case 'T' :
			//center
			glTranslatef(size/2.0 - r, 0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			//top
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-size + 2*r, 0.0, -size/2.0 + r);
			glprimitive_cylinder(r, size - 2*r);	
			//caps
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
		break;
		case 'U' :
			//Curve
			glRotatef(-90, 1.0, 0.0, 0.0);
			glRotatef(90, 0.0, 0.0, 1.0);
			glTranslatef(-size/2.0, -size/2.0 + r, 0.0);
			glprimitive_half_torus1(size/2.0 - 2*r, r);
			//Cylinder
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, size/2.0 - r, -size/2.0 + r);
			gluSphere(s, r, d, d);
			glprimitive_cylinder(r, size/2.0 - r);
			glTranslatef(0.0, -size + 2*r, 0.0);
			glprimitive_cylinder(r, size/2.0 - r);
			gluSphere(s, r, d, d);
		break;
		case 'V' :
			glTranslatef(size/2.0 - r, 0.0, r/2.0);
			glPushMatrix();
				glRotatef(-24, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size - r);
				//cap
				glTranslatef(0.0, 0.0, size - r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glPushMatrix();
				glRotatef(24, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size - r);
				//cap
				glTranslatef(0.0, 0.0, size - r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			//Top cap
			gluSphere(s, r, d, d);
		break;
		case 'W' :
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			//caps
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(size - 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			gluSphere(s, r, d, d);
			//Right
			glprimitive_cylinder(r, size - 2*r);	
			//Middle 
			glPushMatrix();
				glRotatef(-45, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0 + r);
			glPopMatrix();	
			glTranslatef(-size + 2*r, 0.0, 0.0);
			glPushMatrix();
				glRotatef(45, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0 + r);
			glPopMatrix();
			//Middle cap
			glTranslatef(size/2.0 - r, 0.0, size/2.0 - r);
			gluSphere(s, r, d, d);
		break;
		case 'X' :
			glTranslatef(0.0, 0.0, r);
			glRotatef(45, 0.0, 1.0, 0.0);
			GLdouble l = sqrt(2*size*size) - 3*r;
			glprimitive_cylinder(r, l);	
			glPushMatrix();
				glTranslatef(0.0, 0.0, l);
				gluSphere(s, r, d, d);
				glTranslatef(0.0, 0.0, -l);
				glRotatef(180, 1.0, 0.0, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glTranslatef(-size/2.0 - r/2.0, 0.0, size/2.0 + r/2.0);
			glRotatef(90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, l);	
			glPushMatrix();
				glTranslatef(0.0, 0.0, l);
				gluSphere(s, r, d, d);
				glTranslatef(0.0, 0.0, -l);
				glRotatef(180, 1.0, 0.0, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
		break;
		case 'Y' :
			glTranslatef(size/2.0 - r, 0.0, r);
			glRotatef(180, 1.0, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size/2.0 + r);
			glprimitive_cylinder(r, size/2.0 - r);
			glPushMatrix();
				glRotatef(135, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0);
				glTranslatef(0.0, 0.0, size/2.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glPushMatrix();
				glRotatef(-135, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size/2.0);
				glTranslatef(0.0, 0.0, size/2.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
		break;
		case 'Z' :
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-size + r, 0.0, -r);
			//Left
			glTranslatef(0.0, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);	
			gluSphere(s, r, d, d);
			//caps
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(size - 2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 2*r);
			gluSphere(s, r, d, d);
			//Right
			glprimitive_cylinder(r, size - 2*r);	
			//Middle
			glTranslatef(0.0, 0.0, 0.0);
			glRotatef(-45, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size + 1.5*r);	
		break;
		case '0' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glprimitive_torus(size/2.0 - 2*r, r);
			glRotatef(90, 1.0, 1.0, 0.0);
			glTranslatef(0.0, 0.0, -size/2.0 + r);
			glprimitive_cylinder(r, size - 2*r);
		break;
		case '1' :
			glPushMatrix();
				//center
				glTranslatef(size/2.0 - r, 0.0, r);
				glprimitive_cylinder(r, size - 2*r);
				//caps
				glTranslatef(0.0, 0.0, size - 2*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			//bottom bar
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-r, 0.0, r);
			glprimitive_cylinder(r, size - 4*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
			//top bar
			glTranslatef(-size + 2*r, 0.0, -size/2.0 - r);
			glprimitive_cylinder(r, size/2.0 - 2*r);
			gluSphere(s, r, d, d);
		break;
		case '2' :
			glRotatef(-90, 1.0, 0.0, 0.0);
			glPushMatrix();
				//Cap for curve
				glRotatef(-90, 1.0, 0.0, 0.0);
				glTranslatef(size/2.0 - 3.5*r, 0.0, -size + 3.5*r);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			//Curve and diagonal stick
			glPushMatrix();
				glRotatef(-90, 0.0, 0.0, 1.0);
				glTranslatef(size - 3.5*r, size/2.0 - r, 0.0);
				glprimitive_half_torus1(1.5*r, r);
				glRotatef(52, 0.0, 0.0, 1.0);
				glprimitive_half_torus1(1.5*r, r);
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, 2.5*r, 0.0);
				glprimitive_cylinder(r, size/2.0 + r);
			glPopMatrix();	
			//Bottom
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(0.0, -r, r);
			glprimitive_cylinder(r, size - 4*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
		break;
		case '3' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0 - r, 3*r, 0.0);
			glPushMatrix();
				//lower curve
				glRotatef(-45, 0.0, 0.0, 1.0);
				glprimitive_half_torus1(r, r);
				//cap
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, -2*r,  0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glprimitive_half_torus1(r, r);
			glTranslatef(0, size - 6*r, 0.0);
			glprimitive_half_torus1(r, r);
			glPushMatrix();
				//upper curve
				glRotatef(45, 0.0, 0.0, 1.0);
				glprimitive_half_torus1(r, r);
				//cap
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, 2*r,  0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();
			//center
			glTranslatef(-r, -2*r, 0.0);
			glRotatef(90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, r);
			glRotatef(180, 1.0, 0.0, 0.0);
			gluSphere(s, r, d, d);
		break;
		case '4' :
			glTranslatef(size - 3*r, 0.0, r);
			glprimitive_cylinder(r, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(-size + 4*r, 0.0, 0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size/2.0 + r);
			glprimitive_cylinder(r, size/2.0 - r);
			gluSphere(s, r, d, d);
			glRotatef(90, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size - 4*r);
		break;
		case '5' :
			//bars
			glPushMatrix();
				glTranslatef(r, 0.0, size/2.0);
				glprimitive_cylinder(r, size/2.0 - r);
				gluSphere(s, r, d, d);
				glTranslatef(0.0, 0.0, size/2.0 - r);
				gluSphere(s, r, d, d);
				glRotatef(90, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size - 5*r);
				glTranslatef(0.0, 0.0, size - 5*r);
				gluSphere(s, r, d, d);
				glTranslatef(size/2.0 - r, 0.0, -r);
				gluSphere(s, r, d, d);
				glTranslatef(0.0, 0.0, -size + 6*r);
				glprimitive_cylinder(r, size - 6*r);
				glTranslatef(size/2.0 - r, 0.0, 0.0);
				glprimitive_cylinder(r, size - 6*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(90, 1.0, 0.0, 0.0);
			//curve
			glTranslatef(size/2.0, 3*r, 0.0);
			glprimitive_half_torus1(r, r);
		break;
		case '6' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0 + r, size/2.0, 0.0);
			glPushMatrix();
				glTranslatef(0.0, -2*r, 0.0);
				glprimitive_torus(r, r);
			glPopMatrix();	
			glPushMatrix();
				glTranslatef(0.0, size/2.0 - 3*r, 0.0);
				glRotatef(90, 0.0, 0.0, 1.0);
				glprimitive_half_torus1(r, r);
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, 2*r, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(2*r, 0.0, -2*r);
			glprimitive_cylinder(r, 4*r);
		break;
		case '7' :
			glPushMatrix();
				glTranslatef(2.5*r, 0.0, r);
				glRotatef(30, 0.0, 1.0, 0.0);
				glprimitive_cylinder(r, size - r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(90, 0.0, 1.0, 0.0);
			glTranslatef(-size + r, 0.0, r);
			glprimitive_cylinder(r, size - 4*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
		break;
		case '8' :
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0 - r, 3*r, 0.0);
			glprimitive_torus(r, r);
			glTranslatef(0, size - 6*r, 0.0);
			glprimitive_torus(r, r);
		break;
		case '9' :
			glPushMatrix();
				glTranslatef(size/2.0 + r, 0.0, r);
				glprimitive_cylinder(r, size - 4*r);
				gluSphere(s, r, d, d);
			glPopMatrix();
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0 - r, size - 3*r, 0.0);
			glprimitive_torus(r, r);
		break;
		case '?':
			//The dot
			glTranslatef(size/2.0 - r, 0.0, r);
			gluSphere(s, r, d, d);
			//The vertical bar
			glTranslatef(0.0, 0.0, 2*r);
			glprimitive_cylinder(r, 2*r);
			gluSphere(s, r, d, d);
			//The curve
			glTranslatef(0.0, 0.0, size/2.0 - r);
			glRotatef(-90, 1.0, 0.0, 0.0);
			glprimitive_half_torus1(r, r);
			glRotatef(-90, 0.0, 0.0, 1.0);
			glprimitive_half_torus1(r, r);
			glPushMatrix();	
				glRotatef(-90, 0.0, 1.0, 0.0);
				glTranslatef(0.0, -2*r, 0.0);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			
			glTranslatef(-2*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
		break;	
		case '!' :
			//The dot
			glTranslatef(size/2.0 - r, 0.0, r);
			gluSphere(s, r, d, d);
			//The stick
			glTranslatef(0.0, 0.0, 3*r);
			glprimitive_cylinder(r, size - 5*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 5*r);
			gluSphere(s, r, d, d);
		break;
		case '.' :
			glTranslatef(size/2.0 - r, 0.0, r);
			gluSphere(s, r, d, d);
		break;
		case ':' :
			glTranslatef(size/2.0 - r, 0.0, 2*r);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size - 5*r);
			gluSphere(s, r, d, d);
		break;
		case '(':
			glRotatef(90, 1.0, 0.0, 0.0);
			glRotatef(180, 0.0, 1.0, 0.0);
			glTranslatef(-size/2.0, size/2.0, 0.0);
			glScalef(0.6, 1.0, 1.0);
			glprimitive_half_torus1((size - 4*r)/2.0, r);
			glTranslatef(0.0, size/2.0 - r, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, -size + 2*r, 0.0);
			gluSphere(s, r, d, d);
		break;
		case ')':
			glRotatef(90, 1.0, 0.0, 0.0);
			glTranslatef(size/2.0 - 2*r, size/2.0, 0.0);
			glScalef(0.6, 1.0, 1.0);
			glprimitive_half_torus1((size - 4*r)/2.0, r);
			glTranslatef(0.0, size/2.0 - r, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, -size + 2*r, 0.0);
			gluSphere(s, r, d, d);
		break;
		case ' ' :
		break;
		case '/':
			glTranslatef(0.0, 0.0, r);
			glRotatef(45, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size);
			gluSphere(s, r, d, d);
		break;
		case '\\':
			glTranslatef(size - 2*r, 0.0, 1*r);
			glRotatef(-45, 0.0, 1.0, 0.0);
			glprimitive_cylinder(r, size);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, size);
			gluSphere(s, r, d, d);
		break;
		case '=' :
			glTranslatef(r, 0.0, 3*r);
			glRotatef(90, 0.0, 1.0, 0.0);
			gluSphere(s, r, d, d);
			glprimitive_cylinder(r, size - 4*r);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
			glTranslatef(-size + 6*r, 0.0, 0.0);
			gluSphere(s, r, d, d);
			glTranslatef(0.0, 0.0, -size + 4*r);
			gluSphere(s, r, d, d);
			glprimitive_cylinder(r, size - 4*r);
		break;
		case '+':
			glPushMatrix();
				glTranslatef(r, 0.0, size/2.0);
				glRotatef(90, 0.0, 1.0, 0.0);
				gluSphere(s, r, d, d);
				glprimitive_cylinder(r, size - 4*r);
				glTranslatef(0.0, 0.0, size - 4*r);
				gluSphere(s, r, d, d);
			glPopMatrix();	
			glTranslatef(size/2.0 - r, 0.0, 2*r);
			gluSphere(s, r, d, d);
			glprimitive_cylinder(r, size - 4*r);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
		break;
		case '-':
			glTranslatef(r, 0.0, size/2.0);
			glRotatef(90, 0.0, 1.0, 0.0);
			gluSphere(s, r, d, d);
			glprimitive_cylinder(r, size - 4*r);
			glTranslatef(0.0, 0.0, size - 4*r);
			gluSphere(s, r, d, d);
		break;
		case '\n' :
		break;
		default:
		break;
	}
	glPopMatrix();
	gluDeleteQuadric(s);
}

inline GLdouble glfont_char_width(GLdouble size)
{
	return size + WIDTH;
}

inline double glfont_char_height(GLdouble size)
{
	return size + WIDTH;
}

void 
glfont_draw_string(char *s, GLdouble size)
{
	GLdouble r = WIDTH;
	
	glPushMatrix();
	glTranslatef(r, 0.0, 0.0);
	glRotatef(-90, 1.0, 0.0, 0.0);
	glPushMatrix();
		while (*s)
		{
			glfont_draw_char(*s, size);
			if (*s == '\n')
			{
				glPopMatrix();
				glTranslatef(0.0, 0.0, -glfont_char_height(size));
				glPushMatrix();
			}
			else
			{
				glTranslatef(glfont_char_width(size), 0.0, 0.0);
			}
			++s;
		}
	glPopMatrix();
	glPopMatrix();
}

void 
glfont_test(GLdouble size, int x, int y)
{
	static GLdouble last_size = 0;

	static GLuint list = 0;
	if (last_size != size)
	{
		if (list == 0)
			list = glGenLists(1);
		glNewList(list, GL_COMPILE);
				glfont_draw_string(font_test_str, size);
		glEndList();	

		last_size = size;
	}
	glCallList(list);
	
	//Draw selector
	static const GLfloat diff_g[] = {0.5, 0.5, 0.5, 1.0};
	static const GLfloat spec_g[] = {0.1, 0.1, 0.1, 1.0};
	static const GLfloat shinny_g = 1;
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff_g);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec_g);
	glMaterialf(GL_FRONT, GL_SHININESS, shinny_g);
	glprimitive_box(glfont_char_width(size)*x, -glfont_char_height(size)*y, 
					-WIDTH, size, size, WIDTH);
}

char glfont_test_char(int x, int y)
{
	int i = y * (GLFONT_TEST_WIDTH + 1) + x;
	return font_test_str[i];
}

bool glfont_is_test_char(char c, int *x, int *y)
{
	//yes i should probably do this with a string function
	
	bool r = false;
	int tx = 0; int ty = 0;
	char *temp = font_test_str; 

	c = toupper(c);
	while (true)
	{
		//Reached a \n. Increament line and move on
		if (tx >= GLFONT_TEST_WIDTH)
		{
			tx = 0;
			++ty;
			++temp;
		}
		
		//End of characters, the given charecter is not valid
		if (ty >= GLFONT_TEST_HEIGHT)
			break;
		
		//Found the given character
		if (c == *temp)
		{
			r = true;
			*x = tx;
			*y = ty;
			break;
		}

		++temp;
		++tx;
	}

	return r;
}
