/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file glfont.h 
 *
 *  These files are responsible for the font.
 */
#ifndef GL_FONT_H
#define GL_FONT_H

#ifdef WIN32
#include <windows.h>
#endif /* WIN32 */

#include <GL/gl.h>
#include <stdbool.h>

/** The width (number of characters) of the font test */
#define GLFONT_TEST_WIDTH 8
/** The height (rows of characters) of the font test */
#define GLFONT_TEST_HEIGHT 6

/** Draws a single character.
 *
 *  @param c The character to draw.
 *  @param size The font size.
 */
inline void 
glfont_draw_char(char c, GLdouble size);

/** Draws a string.
 *
 *  @param s The string to draw.
 *  @param size The font size.
 */
inline void
glfont_draw_string(char *s, GLdouble size);

/** Returns how much space is used by a character for the given sized font 
 *  @param size The size of the font
 *  @return The character width
 */
inline GLdouble 
glfont_char_width(GLdouble size);

/** Returns how much space is used by a character for the given sized font 
 *  @param size The size of the font
 *  @return The character height
 */
GLdouble 
glfont_char_height(GLdouble size);

/** Draws all of the characters 
 *  
 *  Draws all the chararacters of the font. Also draws 
 *  a selector behind the given chararacter
 *   
 *  @param size The size of the font
 *  @param x The x location of the selector
 *  @param y The y location of the selector
 */
void 
glfont_test(GLdouble size, int x, int y);

/** Returns the font character that is selected by glfont_test at x,y
 * 
 *  @param x x value pased to glfont_test
 *  @param y y value passed to glfont_test
 *  @return Returns the selected character
 */
char glfont_test_char(int x, int y);

/** Determines if the given character is in the font and returns
 *  its x/y location in the font test.
 * 
 *  @param c The character to check
 *  @param x Pointer to the location to return x
 *  @param y Pointer to the location to return y
 *  @return Returns true if the charater is valid. The x and y location
 *  		are returned in x, y only if true is returned otherwise these
 *  		values are untouched
*/
bool glfont_is_test_char(char c, int *x, int *y);

#endif /* GL_FONT_H */
