/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file glprimitive.h 
 *
 *  Draw curved primitives with splines.
 */
#ifndef GL_PRIMITIVE_H
#define GL_PRIMITIVE_H

#ifdef WIN32
#include <windows.h>
#endif /* WIN32 */

#include <GL/gl.h>
#include <GL/glu.h>

#ifdef WIN32
#include "glui.h"
#endif /* WIN32 */


/** Sets how detailed primitives are 
 * 	@param d The detail
 */
void 
glprimitive_set_detail(size_t d);

/** Gets how detailed primitives are 
 * 	@return The detail
 */
size_t
glprimitive_get_detail();

/** Draws a box.
 *
 *  The box is drawn with origin x,y,z. It moves away from the orign
 *  by w along positive x, by h along positive y and by d along positive z.
 * 
 *  @param x X origin
 *  @param y Y origin
 *  @param z Z origin
 *  @param w Width
 *  @param h Height
 *  @param d Depth
 */
void
glprimitive_box(GLdouble x, GLdouble y, GLdouble z, 
				 GLdouble w, GLdouble h, GLdouble d);

/** Draws a disk.
 *
 *  The fisk will be on the x-y plane with normal along positive z
 *
 *  @param radius The radius
 */
void
glprimitive_disk(GLdouble radius);

/** Draws a cone
 *
 *  The cone moves outward from the origin along positive z. It is oriented so 
 *  that the whole in the center is on the x-y plane. The major radius is the
 *  radius of the opening on positive z and the minor radius is the radius at 
 *  the origin.
 *  
 *  @param r1 The major radius of the cone
 *  @param r2 The minor radius of the cone
 *  @param len The length
 */
void
glprimitive_cone(GLdouble r1, GLdouble r2, GLdouble len);

/** Draws a cylinder
 *
 *  The cylinder will be centered at the origin moving outward along positive z.
 *  It is oriented so that the whole in the center is on the x-y plane.
 *  
 *  @param radius The radius
 *  @param len The length
 */
void
glprimitive_cylinder(GLdouble radius, GLdouble len);

/** Draws a sphere
 *  @param radius The radius
 */
void
glprimitive_sphere(GLdouble radius);

/** Draws a half sphere
 *
 *  The back (open part) of the sphere will be on the x-y plane with 
 *  the drawn part along positive z
 *
 *  @param radius The radius
 */
void
glprimitive_half_sphere(GLdouble radius);

/** Draws a torus.
 *
 *  The torus will be centered at the origin. It is oriented so that
 *  the whole in the center is on the x-y plane.
 * 
 *  @param r1 The major radius of the torus. The radius of the hole.
 *  @param r2 The minor radius of the torus. The radius of the curved part.
 */
void
glprimitive_torus(GLdouble r1, GLdouble r2);

/** Draws a half torus, verions 1
 * 
 *  Draws a torus which is shapred like a u. The parts that are on the
 *  positive side of z = 0 plane.
 * 
 *  @param r1 The major radius of the torus. The radius of the hole.
 *  @param r2 The minor radius of the torus. The radius of the curved par.
 */
void
glprimitive_half_torus1(GLdouble r1, GLdouble r2);

/** Draws a half torus, verions 1
 * 
 *  The parts that are on the positive side of x = 0 plane.
 * 
 *  @param r1 The major radius of the torus. The radius of the hole.
 *  @param r2 The minor radius of the torus. The radius of the curved par.
 */
void
glprimitive_half_torus2(GLdouble r1, GLdouble r2);

/** Draws all the primtitives, with given radius and detail
 *  @param r Radius
 *  @param d detail
 */
void 
glprimitive_test(GLdouble r, size_t d);

/** Draws a clock 
 * 
 *  @param size How big to draw the clock.
 *  @param d Detail to use
 */
void
glprimitive_clock(GLdouble size, size_t d);

/** Draws a mug
 * 
 *  @param size How big to draw the mug.
 *  @param d Detail to use
 */
void
glprimitive_mug(GLdouble size, size_t d);
	
#endif /* GL_PRIMITIVE_H */
