/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */
#ifdef WIN32
#include <windows.h>
#endif /* WIN32 */

#include <GL/glut.h>

#include <stdlib.h>
#include <stdbool.h>
#include <time.h>
#include <getopt.h>
#include <string.h>

#include "battleship_view.h"
#include "battleship_logic.h"
#include "battleship_model.h"

#define TQUEUE_LEN 5
struct timer_struct
{
	bool (*func)(void);
	time_t delay;
	bool stopped;
};
static struct timer_struct timers[TQUEUE_LEN];
static time_t timerS, timerE;

static void timer(int val);

void
battleship_view_refresh()
{
	glutPostRedisplay();
}

void
battleship_view_swap_buffers()
{
	glutSwapBuffers();
}

void 
battleship_logic_quit()
{
	exit(0);
}

void
battleship_logic_start_timer(time_t delay, bool (*func)(void))
{
	bool empty = false;
	
	//check for empty/full
	if (timerS == timerE)
	{
		if (timers[timerS].func != NULL)
		{
			//DPRINT("Error: Timmer queue full");
			return;
		}
		else
			empty = true;
	}
	//	DPRINT("9 %i, %i, %i\n", timerS, timerE, func);

	timers[timerE].func = func;
	timers[timerE].delay = delay;
	timerE = (timerE + 1) % TQUEUE_LEN;
	
	if (empty) timer(0);
}

void
timer(int val)
{
	(void) val;

	if (timers[timerS].stopped || !timers[timerS].func())
	{
		timers[timerS].func = NULL;
		timers[timerS].delay = 0;
		timers[timerS].stopped = false;
		timerS = (timerS + 1) % TQUEUE_LEN;
	}

	if (timerS != timerE)
		glutTimerFunc(timers[timerS].delay, timer, 0);
}

void
battleship_logic_stop_timers()
{
	size_t i;
	for (i = 0; i < TQUEUE_LEN; i++)
	{
		if (timers[i].func != NULL)
			timers[i].stopped = true;
	}
}

void
motion(int x, int y)
{
	battleship_logic_mouse_move(x, y);
}

void
mouse(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)
		battleship_logic_mouse_down(button+1,  glutGetModifiers(), x, y);
	else if (state == GLUT_UP)
		battleship_logic_mouse_up(button+1,  glutGetModifiers(), x, y);
}
  
void
keyboard(unsigned char k, int x, int y)
{
	(void) x;
	(void) y;
	battleship_logic_keypress(k,  glutGetModifiers());
}

void 
specialKeyboard(int k, int x, int y)
{
	(void) x;
	(void) y;

	//This captures the keys up/down/left/right/home and converts them
	//to the vaules used by battleship_logic
	switch (k)
	{
		case 'd': k = 1; break;
		case 'f': k = 2; break;
		case 'e': k = 3; break;
		case 'g': k = 4; break;
		case 'j': k = 5; break;
		default: k = 0; break;
	}
	
	if (k != 0)
		battleship_logic_keypress(k,  glutGetModifiers());
}

void
resize(int w, int h)
{
	battleship_view_resize(w, h);
}

time_t last_time;
void
idle()
{
	if (time(0) - last_time > 0)
		battleship_view_refresh();
	last_time = time(0);
}

size_t seed;
int test = 0;

int
main(int argc, char** argv)
{
	last_time = time(0);
	test = 0;
	seed = time(0);
	while (1)
	{
		static struct option long_options[] =
		{
			{"test", required_argument, NULL, 0},
			{"seed", required_argument, NULL, 0},
			{0, 0, 0, 0}
		};
		int option_index = 0;
		int c = getopt_long(argc, argv, "t:s:", long_options, &option_index);
		if (c == -1)
			break;
		switch (c)
		{
			case 0:
				if (!strcmp("seed", long_options[option_index].name))
					seed = atoi(optarg);
				else if (!strcmp("test", long_options[option_index].name))
					test = atoi(optarg);
			break;
			case 't': test = atoi(optarg); break;
			case 's': seed = atoi(optarg); break;
			default:
				abort();
		}
	}

	//their are arguments
	if (optind  < argc)
	{
	}
	
	srand(seed);
	//DPRINT("Rand Seed %zu\n", seed);
	
	int width = 1024;
	int height = 768;

	//this just initizises the timer variables
	timerS = 0;
	timerE = 0;
	size_t i;
	for (i = 0; i < TQUEUE_LEN; i++)
	{
		timers[i].func = NULL;
		timers[i].delay = 0;
		timers[i].stopped = false;
	}

    // intialize glut and main window
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE|GLUT_DEPTH);
    glutInitWindowSize(width, height);
    glutCreateWindow(BATTLESHIP_NAME " " BATTLESHIP_VERSION);
	glutFullScreen();
	
    // initialize callback
    glutDisplayFunc(battleship_view_main);
    glutReshapeFunc(resize);
    glutKeyboardFunc(keyboard);
    glutSpecialFunc(specialKeyboard);
    glutMouseFunc(mouse);
    glutMotionFunc(motion);
    glutPassiveMotionFunc(motion);

	//set things from command line arguments
	if (test >= NUM_TEST_TYPES)
		test = 0;
	do_test = (enum battleship_testtype) test;

	//initilize everything
	battleship_logic_init();

    resize(width, height);
  
    glutMainLoop();
  
    return 0;
}
