/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * I used these sites and book as a references when creating this:
 * 
 * http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
 * http://www.animeimaging.com/asp/PerlinNoise.aspx   
 * Shirley,Peter and RMorley Keith ,R. (2001) Realistic Ray Tracing, 
 * 	2nd edition. A.K. Peters.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file perlin.c 
 *
 *  A perlin noise generator  
 */
#include <math.h>

#include "perlin.h"

/** The current seed for the random number generator */
static int seed = 1;

/** Generates a random number based on x and y.
 *
 *  The same random number will always be returned for the same x and y
 *  when the global value seed remains constant. Seed can be changed to
 *  produce a different sequence.
 *  
 *  @param x x value
 *  @param y y value
 *  @return Returns a random number, same value always returned for same
 *  value of x and y
 */
/* I would like to make this function use rand. The seed thing I made
   doesn't make that much senes. The problem with using rand is how 
   do you get the same number again. The function needs to work so that
   when you always get the same results for the same values of x and y
*/
static double noise2D(int x, int y)
{
   int n;
   n = seed * x + y * 57;
   n = (n << 13) ^ n;
   double res = (double) (1.0 - ( (n * (n * n * 15731 + 789221)
                           + 1376312589) & 0x7fffffff ) / 1073741824.0);
   return res;
}

/** Performs linear interpolation 
 *  @param a Start value
 *  @param b End vaule
 *  @param d Interval size
 *  @return Returns linearly interpolated value between x and y
 */
inline double lin_interp2D(double a, double b, double d)
{
	return  a*(1-d) + b*d;
}

/** Performs cosine interpolation 
 *  @param a Start value
 *  @param b End vaule
 *  @param d Interval size
 *  @return Returns cosine interpolated value between x and y
 */
inline double cos_interp2D(double a, double b, double d)
{
	double t = (1 - cos(d * M_PI)) * 0.5;
	return a*(1 - t) + b*t;
}

/** Performs easy curve interpolation 
 *  @param a Start value
 *  @param b End vaule
 *  @param d Interval size
 *  @return Returns easy curve interpolated value between x and y
 */
inline double ecurve_interp2D(double a, double b, double d)
{
	double f1 = 3.0 * pow(1.0 - d, 2.0) - 2.0 * pow(1.0 - d, 3.0);
  	double f2 = 3.0 * pow(d, 2.0) - 2.0 * pow(d, 3.0);

    return a*f1 + b*f2; 
}

/** Performs interpolation (using one of the interpolation functions)
 *  @param a Start value
 *  @param b End vaule
 *  @param d Interval size
 *  @return Returns interpolated value between x and y
 */
double interpolate2D(double a, double b, double d)
{
	return ecurve_interp2D(a, b, d);	
}

/** Performs smoothing (anitaliasing) for given location.
 *
 *  Samples from points around point and calculate waited average.
 *
 *  @param x x value
 *  @param y y value
 *
 *  @return smoothed value
 */
double smooth2D(int x, int y)
{
    double corners = noise2D(x - 1, y - 1) + noise2D(x + 1, y - 1) +
					 noise2D(x - 1, y + 1) + noise2D(x + 1, y + 1);
    double sides = noise2D(x - 1, y) + noise2D(x + 1, y) +
					noise2D(x, y - 1) + noise2D(x, y + 1);
    double center = noise2D(x, y);
	
    return corners/16.0 + sides/8.0 + center/4.0;
}

/** Generates the base perlin number, applying smoothing and interpolation
 *  @param x The x value
 *  @param y The y value
 *  @return The number
 **/
double gen2D(double x, double y)
{
	//There is probably a better more stable way to do this
	int ix = (int) x;
	double fx = x - ix;
	int iy = (int) y;
	double fy = y - iy;

	//find the four corners
	double x0y0 = smooth2D(ix, iy);
	double x0y1 = smooth2D(ix, iy+ 1);
	double x1y0 = smooth2D(ix + 1, iy);
	double x1y1 = smooth2D(ix + 1, iy + 1);

	//interpolate on x
	double i1 = interpolate2D(x0y0, x1y0, fx); 
	double i2 = interpolate2D(x0y1, x1y1, fx);
	
	//Interpolate on y
	return interpolate2D(i1, i2, fy); 
}

inline void sperlin(int s)
{
	seed = s;
}

double
perlin2D(struct perlin param, int x, int y)
{
	double total = 0.0;
	double freq = param.freq;
	double amp = 1;
	
	unsigned int i = 0;
	for (i = 0; i < param.octaves; i++)
	{
		total += gen2D(x * freq, y * freq) * amp;
		freq *= 2;
		amp *= param.pers;
	}
	if (total < -1.0) total = -1.0;
	if (total > 1.0) total = 1.0;
	
	return total; 
}
