/* Matthew Phillips mephilli 20066500 CS488 A5
 *
 * Copyright (c) 2005 Matthew Phillips
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/** @file psystem.h 
 *
 *  Resposible for particle system.
 */
#ifndef PSYSTEM_H
#define PSYSTEM_H

#ifdef WIN32
#include <windows.h>
#endif /* WIN32 */

#include <GL/gl.h>
#include <GL/glu.h>

#ifdef WIN32
#include "glui.h"
#endif /* WIN32 */

#include <stddef.h>

/** The maximum number of particles */
#define MAX_NUM_PARTICLES 500

/** How many fire colours there are */
#define NUM_PSYSTEM_COLOURS 4 
/** The possible rocket paths */
enum psystem_colour
{
	PSYSTEM_COLOUR_ORANGE,
	PSYSTEM_COLOUR_BLUE,
	PSYSTEM_COLOUR_RAINBOW,
	PSYSTEM_COLOUR_RANDOM,
};
/** String representation of fire colours */
static const char *psystem_colours_s[NUM_PSYSTEM_COLOURS] = 
{
	"Orange ",
	"Blue   ",
	"Rainbow",
	"Random ",
};

/** Structure representng a single particle in the particle system */
struct psystem_particle
{
	/** The size of the particle */
	GLdouble size;
	/** The particles current position */
	GLdouble pos[3];
	/** The direction the particle is traveling */
	GLdouble dir[3];
	/** The current rotation of the particle */
	GLdouble rot[3];
	/** The way the particle is spinning */
	GLdouble spin[3];

	/** How long the particle has left to live */
	size_t life;
};

/** Structure represeting a particle system */
struct psystem
{
	/** The radius of particle origin */
	GLdouble radius;
	/** The radius of particle destination */
	GLdouble radius2;
	/** The distance from the origin to the destination */
	GLdouble len;

	/** What colour scheme to use */
	enum psystem_colour colour;

	/** The particles */
	struct psystem_particle particles[MAX_NUM_PARTICLES];
	/** Current number of particls */
	size_t num_particles;
};

/** Initilizes a particle system with the given parameters 
 *  @param p The particle system
 *  @param r1 The orign radius
 *  @param r2 The destination radius
 *  @param len The distance between the orign and destination
 *  @param c The colour scheme to use
 */
void
psystem_init(struct psystem *p, GLdouble r1, GLdouble r2, GLdouble len,
			 enum psystem_colour c);

/** Resets a particle system.
 *  @param p The particle system
 */
void
psystem_reset(struct psystem *p);

/** Addvances all of the particles in the system and generates new particles
 *  @param p The particle system 
 */
void
psystem_advance(struct psystem *p);

/** Draws all the particles in the system. 
 *  @param p The particle system 
 */
void 
psystem_draw(struct psystem *p);

#endif /* PSYSTEM_H */
